Package: cellmig
Type: Package
Title: Uncertainty-aware quantitative analysis of high-throughput live
        cell migration data
Version: 1.0.0
Authors@R: c(
    person("Simo", "Kitanovski", email = "simokitanovski@gmail.com", 
    role = c("aut", "cre"), comment=c(ORCID="0000-0003-2909-5376")))
Description: High-throughput cell imaging facilitates the analysis of cell 
    migration across many wells treated under different biological conditions. 
    These workflows generate considerable technical noise and biological 
    variability, and therefore technical and biological replicates are 
    necessary, leading to large, hierarchically structured datasets, i.e., 
    cells are nested within technical replicates that are nested within 
    biological replicates. Current statistical analyses of such data usually 
    ignore the hierarchical structure of the data and fail to explicitly 
    quantify uncertainty arising from technical or biological variability. 
    To address this gap, we present cellmig, an R package implementing Bayesian 
    hierarchical models for migration analysis. cellmig quantifies condition-
    specific velocity changes (e.g., drug effects) while modeling nested data 
    structures and technical artifacts. It further enables synthetic data 
    generation for experimental design optimization. 
License: GPL-3 + file LICENSE
Depends: R (>= 4.5.0)
Imports: base, ggplot2, ggforce, ggtree, patchwork, ape, methods, Rcpp
        (>= 0.12.0), RcppParallel (>= 5.0.1), reshape2, rstan (>=
        2.18.1), rstantools (>= 2.4.0), stats, utils, scales
Suggests: BiocStyle, knitr, testthat
Encoding: UTF-8
NeedsCompilation: yes
biocViews: SingleCell, CellBiology, Bayesian, ExperimentalDesign,
        Software, BatchEffect, Regression, Clustering
BugReports: https://github.com/snaketron/cellmig/issues
URL: https://github.com/snaketron/cellmig
RoxygenNote: 7.3.1
VignetteBuilder: knitr
Biarch: true
LinkingTo: BH (>= 1.66.0), Rcpp (>= 0.12.0), RcppEigen (>= 0.3.3.3.0),
        RcppParallel (>= 5.0.1), rstan (>= 2.18.1), StanHeaders (>=
        2.18.0)
SystemRequirements: GNU make
git_url: https://git.bioconductor.org/packages/cellmig
git_branch: RELEASE_3_22
git_last_commit: e77030e
git_last_commit_date: 2025-10-29
Repository: Bioconductor 3.22
Date/Publication: 2025-10-30
Packaged: 2025-10-30 20:52:11 UTC; biocbuild
Author: Simo Kitanovski [aut, cre] (ORCID:
    <https://orcid.org/0000-0003-2909-5376>)
Maintainer: Simo Kitanovski <simokitanovski@gmail.com>

# Generated by roxygen2: do not edit by hand
export(cellmig)
export(get_dose_response_profile)
export(gen_partial)
export(gen_full)
export(get_ppc_violins)
export(get_ppc_means)
export(get_pairs)
export(get_violins)
export(get_groups)
import(utils)
import(Rcpp)
import(methods)
import(ggplot2)
importFrom(ape, as.phylo)
importFrom(ape, prop.clades)
importFrom(ape, is.rooted)
importFrom(ggforce, geom_sina)
importFrom(rstan, sampling)
importFrom(rstan, summary)
importFrom(rstan, extract)
importFrom(rstantools, rstan_config)
importFrom(reshape2, acast)
importFrom(reshape2, melt)
importFrom(RcppParallel, RcppParallelLibs)
importFrom(stats, dist, hclust, aggregate, rgamma, rnorm)
importFrom(scales, pretty_breaks)
importFrom(ggtree, ggtree, geom_point2, geom_tippoint, geom_tiplab, 
           geom_nodelab, layout_rectangular, revts)
importFrom(patchwork, plot_layout, plot_annotation)
useDynLib(cellmig, .registration = TRUE)

# cellmig: Uncertainty-aware quantitative analysis of high-throughput live cell migration data

# Background

High-throughput tracking of cells with time-lapse microscopy followed by
the acquisition of images at ﬁxed time intervals facilitates the
analysis of cell migration across many wells treated under different
biological conditions. These workflows generate considerable technical
noise and biological variability, and therefore technical and biological
replicates are necessary, leading to large, hierarchically structured
datasets, i.e., cells are nested within technical replicates that are
nested within biological replicates.

Current statistical analyses of such data usually ignore the
hierarchical structure of the data and fail to explicitly quantify
uncertainty arising from technical or biological variability. To address
this gap, we present **cellmig**, an R package implementing
Bayesian hierarchical models for migration analysis.
**cellmig** quantifies condition-specific velocity changes
(e.g., drug effects) while modeling nested data structures and technical
artifacts, providing uncertainty-aware estimates through credible
intervals.

Furthermore, **cellmig** includes functionality for simulating
synthetic datasets. These simulations are invaluable for experimental
planning, allowing researchers to assess how different experimental
designs (e.g., varying numbers of biological replicates, technical
replicates, or cells per well) affect the precision of treatment effect
estimates.


# Installation
To install this package, start R (version "4.5") and enter:

```{r, eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("cellmig")
```

Case studies are provided in the directory /vignettes


# How to cite
bioRxiv preprint doi: https://doi.org/10.1101/2025.06.12.659342

get_groups <- function(x) {
  check_generic(y = x)
  m <- x$posteriors$delta_t[, c("group_id", "group", "compound", "dose")]
  return(m)
}

get_pairs <- function(x, groups, exponentiate) {
  check_generic(y = x)
  check_logical_val(y = exponentiate, par = "exponentiate")
  
  gmap <- get_groups(x = x)
  if(missing(groups)) {
    warning("groups not specified, we will use all groups")
    groups <- gmap$group
  }
  if(any(is.na(groups))|any(is.nan(groups))) {
    warning("groups not specified, we will use all groups")
    groups <- gmap$group
  }
  check_character_vec(y = groups, par = "groups")
  if(length(groups)==1) {
    stop("only one treatment groups provided, length(groups)>1")
  }
  p <- extract(x$fit, par = "mu_group")$mu_group
  if(ncol(p)==1) {
    stop("only one treatment group: nothing to compare")
  }
  if(all(groups %in% gmap$group)==FALSE) {
    stop("unknown group in groups")
  }
  
  gmap <- gmap[gmap$group %in% groups,]
  gmap <- gmap[match(groups, gmap$group),]
  
  idx <- expand.grid(i = seq_len(nrow(gmap)), j = seq_len(nrow(gmap)))
  
  ds <- do.call(rbind, mapply(FUN = function(i, j) {
    d <- p[, gmap$group_id[i]] - p[, gmap$group_id[j]]
    pmax <- get_pmax(d)
    d_M <- mean(d)
    d_HDI <- get_hdi(vec = d, hdi_level = 0.95)
    return(data.frame(group_id_x = gmap$group_id[i], 
                      group_id_y = gmap$group_id[j], 
                      group_x = gmap$group[i], 
                      group_y = gmap$group[j], 
                      compound = gmap$compound[i], 
                      dose = gmap$dose[i],
                      rho_M = d_M,
                      rho_L95 = d_HDI[1],
                      rho_H95 = d_HDI[2],
                      rho_M_exp = exp(d_M),
                      rho_L95_exp = exp(d_HDI[1]),
                      rho_H95_exp = exp(d_HDI[2]),
                      pmax = pmax))}, 
    idx$i, idx$j, SIMPLIFY = FALSE))
  ds$group_x <- factor(x = ds$group_x, levels = groups)
  ds$group_y <- factor(x = ds$group_y, levels = groups)
  
  g_pi <- ggplot(data = ds)+
    geom_tile(aes(y = group_x, x = group_y, fill = pmax), col = "white")+
    geom_text(aes(y = group_x, x = group_y, 
                  label = round(x = pmax, digits = 2)), size = 2)+
    scale_fill_gradient(name = expression(pi), low = "white",high = "darkgray")+
    theme_bw(base_size = 10)+
    theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
    xlab(label = '')+
    ylab(label = '')
  
  if(exponentiate==FALSE) {
    g <- ggplot(data = ds)+
      geom_tile(aes(y = group_x, x = group_y, fill = rho_M), col = "white")+
      geom_text(aes(y = group_x, x = group_y, 
                    label = round(x = rho_M, digits = 1)), size = 2)+
      scale_fill_distiller(name = expression(rho), palette = "Spectral")+
      scale_radius(name = expression(rho))+
      theme_bw(base_size = 10)+
      theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
      xlab(label = '')+
      ylab(label = '')
  } 
  else {
    g <- ggplot(data = ds)+
      geom_tile(aes(y = group_x, x = group_y, fill = rho_M_exp), col = "white")+
      geom_text(aes(y = group_x, x = group_y, 
                    label = round(x = rho_M_exp, digits = 1)), size = 2)+
      scale_fill_distiller(name = expression(rho*"'"), palette = "Spectral")+
      scale_radius(name = expression(rho*"'"))+
      theme_bw(base_size = 10)+
      theme(axis.text.x = element_text(angle = 90, vjust = 0.5, hjust=1))+
      xlab(label = '')+
      ylab(label = '')
  }
  return(list(ds = ds, plot_rho = g, plot_pi = g_pi))
}

get_violins <- function(x, from_groups, to_group, exponentiate) {
  check_generic(y = x)
  check_logical_val(y = exponentiate, par = "exponentiate")
  check_length_one(y = to_group, par = "to_group")

  p <- extract(x$fit, par = "mu_group")$mu_group
  if(ncol(p)==1) {
    stop("only one treatment group: nothing to compare")
  }
  
  gmap <- get_groups(x = x)
  if(all(from_groups %in% gmap$group)==FALSE) {
    stop("unknown group in from_groups")
  }
  if(all(to_group %in% gmap$group)==FALSE) {
    stop("unknown group in to_group")
  }
  
  gmap_from <- gmap[gmap$group %in% from_groups,]
  gmap_to <- gmap[gmap$group %in% to_group,]
  
  idx <- expand.grid(i = seq_len(nrow(gmap_from)), j = seq_len(nrow(gmap_to)))
  ds <- do.call(rbind, mapply(function(i, j) {
    d <- p[, gmap_from$group_id[i]] - p[, gmap_to$group_id[j]]
    pmax <- get_pmax(d)
    return(data.frame(rho = d,
                      group_id_x = gmap_from$group_id[i], 
                      group_id_y = gmap_to$group_id[j], 
                      group_x = gmap_from$group[i], 
                      group_y = gmap_to$group[j], 
                      compound = gmap_from$compound[i], 
                      dose = gmap_from$dose[i],
                      pmax = pmax))
  }, idx$i, idx$j, SIMPLIFY = FALSE))
  
  ds$contrast <- paste0(ds$group_x, "-vs-", ds$group_y) 
  ds$contrast <- factor(x = ds$contrast, 
                        levels = paste0(from_groups, "-vs-", to_group))
  ds_pmax <- ds[duplicated(ds[, c("group_x", "group_y")])==FALSE,]
  
  if(exponentiate==FALSE) {
    g <- ggplot(data = ds)+
      facet_wrap(facets = ~compound, scales = "free_x", nrow = 1)+
      geom_hline(yintercept = 0, linetype = "dashed")+
      geom_violin(aes(x = dose, y = rho), 
                  col = "steelblue", fill = "steelblue", alpha = 0.7)+
      geom_text(data = ds_pmax, aes(x = dose, y = max(ds$rho)+0.15, 
                    label = round(x = pmax, digits = 2)), size = 2.25)+
      theme_bw(base_size = 10)+
      theme(strip.text.x = element_text(margin = margin(0.03,0,0.03,0,"cm")))+
      xlab(label = 'Dose')+
      ylab(label = expression("LFC ("*rho*")"))+
      ggtitle(label = paste0("treatments / ", to_group))
  } else {
    g <- ggplot(data = ds)+
      facet_wrap(facets = ~compound, scales = "free_x", nrow = 1)+
      geom_hline(yintercept = 1, linetype = "dashed")+
      geom_violin(aes(x = dose, y = exp(rho)), 
                  col = "steelblue", fill = "steelblue", alpha = 0.7)+
      geom_text(data = ds_pmax, aes(x = dose, y = max(exp(ds$rho))+0.15, 
                    label = round(x = pmax, digits = 2)), size = 2.25)+
      theme_bw(base_size = 10)+
      theme(strip.text.x = element_text(margin = margin(0.03,0,0.03,0,"cm")))+
      xlab(label = 'Dose')+
      ylab(label = expression("FC ("*rho*"')"))+
      ggtitle(label = paste0("treatments / ", to_group))
  }
  return(list(ds = ds, plot = g))
}

###### testing downstream #######

test_that("downstream functions with invalid inputs", {
  data("d", package = "cellmig")
  d <- d[d$compound %in% c("C1", "C2"),]
  d <- d[d$plate == "1", ]
  o <- suppressWarnings(cellmig(x = d, control = list(mcmc_chains = 1,
                                                      mcmc_warmup = 200,
                                                      mcmc_steps = 500)))
  
  groups <- get_groups(x = o)
  expect_no_error(get_groups(x = o))
  expect_error(get_groups())
  expect_error(get_groups(x = NULL))
  
  expect_no_error(get_violins(x = o, exponentiate = TRUE, 
                              from_group = "C2|D1",to_group = "C2|D2"))
  expect_no_error(get_violins(x = o, exponentiate = FALSE, 
                              from_group = "C2|D1",to_group = "C2|D2"))
  expect_error(get_violins(x = o, from_groups = "C2|D1",to_group = "C2|D2"))
  expect_error(get_violins(x = o, from_groups = "C2|D1",to_group = "C2|D2"))
  
  
  expect_warning(get_pairs(x = o, exponentiate = TRUE))
  expect_warning(get_pairs(x = o, exponentiate = FALSE))
  
  expect_error(get_pairs())
  expect_error(get_pairs(x = o))
  expect_error(get_pairs(x = o, groups = 1))
  expect_error(get_pairs(x = o, groups = NA))
  expect_error(get_pairs(x = o, groups = NULL))
  expect_no_error(get_pairs(x = o, groups = groups$group, exponentiate = TRUE))
  expect_no_error(get_pairs(x = o, groups = groups$group, exponentiate = FALSE))
})

get_dose_response_profile <- function(x,
                                      hc_link = "average",
                                      hc_dist = "euclidean",
                                      groups,
                                      B = 1000) {
  
  check_profile_inputs(x = x, hc_link = hc_link, hc_dist = hc_dist, B = B)
  
  gmap <- get_groups(x = x)
  if(missing(groups)) {
    warning("groups not specified, we will use all groups")
    groups <- gmap$group
  }
  if(any(is.na(groups))|any(is.nan(groups))) {
    warning("groups not specified, we will use all groups")
    groups <- gmap$group
  }
  if(any(is.character(groups)==FALSE)) {
    warning("groups must be characters")
  }
  if(length(groups)==1) {
    stop("only one treatment groups provided, length(groups)>1")
  }
  eg <- x$posteriors$delta_t
  es <- x$posteriors$delta_tp
  if(any(!groups %in% unique(eg$group))) {
    stop("selected groups not found in data")
  }
  eg <- eg[eg$group %in% groups, ]
  es <- es[es$group %in% groups, ]
  
  bt <- get_boot_drp(x = x, hc_dist = hc_dist, hc_link = hc_link, B = B)
  
  tree <- ggtree(bt$main, linetype='solid')+
    geom_point2(mapping = aes(subset=isTip==FALSE),size = 0.5, col = "black")+
    geom_tippoint(size = 2, fill = "white", shape = 21)+
    geom_tiplab(color='black', as_ylab = TRUE, align = TRUE)+
    layout_rectangular()+
    theme_bw(base_size = 10)+
    scale_x_continuous(labels = abs)+
    geom_nodelab(geom='text', color = "#4c4c4c" ,size = 2.75, hjust=-0.2,
                 mapping = aes(label=label,subset=isTip==FALSE))
  tree <- revts(tree)
  
  t <- tree$data
  t <- t[order(t$y, decreasing = FALSE), ]
  tips <- t$label[t$isTip==TRUE]
  
  q <- eg
  q$compound <- factor(q$compound, levels = rev(tips))
  
  g <- ggplot(data = q)+
    facet_grid(compound~., switch = "y")+
    geom_hline(yintercept = 0, linetype = "dashed", col = "gray")+
    geom_point(aes(x = dose, y = mean))+
    geom_errorbar(aes(x = dose, y = mean, ymin = X2.5., ymax = X97.5.),width=0)+
    scale_y_continuous(position = "right", breaks = pretty_breaks(n = 5))+
    theme_bw(base_size = 10)+
    theme(strip.text.y=element_text(margin = margin(0.01,0.01,0.01,0.01,"cm")))
  
  q <- es[es$compound %in% q$compound, ]
  q$compound <- factor(q$compound, levels = rev(tips))
  g2 <- ggplot(data = q)+
    geom_hline(yintercept = 0, linetype = "dashed", col = "gray")+
    facet_wrap(facets = compound~plate, nrow = length(unique(q$compound)), 
               strip.position = "top", scales = "free_y")+
    geom_errorbar(aes(x = dose, y = mean, ymin = X2.5., ymax = X97.5.), 
                  width = 0, alpha = 0.5)+
    geom_line(aes(x = dose, y = mean))+
    geom_point(aes(x = dose, y = mean))+
    scale_y_continuous(position = "right", breaks = pretty_breaks(n = 3))+
    theme_bw(base_size = 10)+
    theme(legend.position = "none", strip.text.y = element_text(
      margin = margin(0.01,0.01,0.01,0.01, "cm")))
  return((tree|g|g2)+plot_annotation(tag_levels = 'A'))
}

get_treatment_profile <- function(x, 
                                  hc_link = "average", 
                                  hc_dist = "euclidean",
                                  groups,
                                  B = 1000) {
  
  check_profile_inputs(x = x, hc_link = hc_link, hc_dist = hc_dist, B = B)
  
  gmap <- get_groups(x = x)
  if(missing(groups)) {
    warning("groups not specified, we will use all groups")
    groups <- gmap$group
  }
  if(any(is.na(groups))|any(is.nan(groups))) {
    warning("groups not specified, we will use all groups")
    groups <- gmap$group
  }
  if(any(is.character(groups)==FALSE)) {
    warning("groups must be characters")
  }
  if(length(groups)==1) {
    stop("only one treatment groups provided, length(groups)>1")
  }
  bt <- get_boot_tp(x = x, hc_dist = hc_dist, hc_link = hc_link,
                    groups = groups, B = B)
  
  tree <- ggtree(bt$main, linetype='solid')+
    geom_point2(mapping = aes(subset=isTip==FALSE),size = 0.5, col = "black")+
    geom_tippoint(size = 2, fill = "white", shape = 21)+
    geom_tiplab(color='black', as_ylab = TRUE, align = TRUE)+
    layout_rectangular()+
    theme_bw(base_size = 10)+
    scale_x_continuous(labels = abs)+
    geom_nodelab(geom='text', color = "#4c4c4c" ,size = 2.75, hjust=-0.2,
                 mapping = aes(label=label,subset=isTip==FALSE))
  
  return(revts(tree))
}

check_profile_inputs <- function(x, 
                                 hc_link, 
                                 hc_dist, 
                                 B) {
  check_generic(y = x)
  
  check_missing(y = hc_link, par = "hc_link")
  check_length_one(y = hc_link, par = "hc_link")
  if(is.character(hc_link)==FALSE) {
    stop("hclust_method must be one of: ward.D, single, complete, 
    average, mcquitty, median, centroid or ward.D2")
  }
  if(!hc_link %in% c("ward.D", "ward.D2", "single", 
                     "complete", "average", "mcquitty",
                     "median", "centroid")) {
    stop("hc_link must be one of: ward.D, single, complete, 
    average, mcquitty, median, centroid or ward.D2")
  }
  
  check_missing(y = hc_dist, par = "hc_dist")
  check_length_one(y = hc_dist, par = "hc_dist")
  if(is.character(hc_dist)==FALSE) {
    stop("hc_dist must be one of: euclidean or manhattan")
  }
  if(!hc_dist %in% c("euclidean", "manhattan")) {
    stop("hc_dist must be one of: euclidean or manhattan")
  }
  
  check_positive_integer(y = B, par = "B")
}

get_boot_drp <- function(x, 
                         hc_dist, 
                         hc_link, 
                         B) {
  
  eg <- x$posteriors$delta_t
  gs <- eg$group_id
  
  q <- acast(data = eg, formula = compound~dose, value.var = "mean")
  hc <- hclust(dist(q, method = hc_dist), method = hc_link)
  main_ph <- as.phylo(x = hc)
  
  # meta
  meta <- x$posteriors$delta_t[, c("group_id", "compound", "dose")]
  meta <- meta[order(meta$group_id, decreasing = FALSE),]
  meta <- meta[meta$group_id %in% gs, ]
  meta$group_id <- NULL
  
  # extract posterior
  e <- extract(x$f, par = "mu_group")$mu_group[, gs]
  e <- e[sample(x = seq_len(nrow(e)), size = min(nrow(e), B),replace = TRUE),]
  
  boot_phs <- lapply(seq_len(nrow(e)), function(i) {
    u <- data.frame(g = seq_len(ncol(e)), mean = e[i, ])
    u <- cbind(u, meta)
    q <- acast(data = u, formula = compound~dose, value.var = "mean")
    hc <- hclust(dist(q, method = hc_dist), method = hc_link)
    return(as.phylo(x = hc))
  })
  
  clades <- prop.clades(phy = main_ph, x = c(boot_phs), part = NULL,
                        rooted = is.rooted(main_ph))
  
  # add bootstrap
  main_ph$node.label <- clades
  
  # b = 0 for these nodes
  na_nodes <- which(is.na(main_ph$node.label))
  if(length(na_nodes)!=0) {
    main_ph$node.label[na_nodes] <- 0
  }
  
  return(list(main_ph = main_ph, boot_ph = c(boot_phs)))
}

get_boot_tp <- function(x, 
                        hc_dist, 
                        hc_link, 
                        groups, B) {
  
  eg <- x$posteriors$delta_t
  
  if(missing(groups)==FALSE) {
    if(any(!groups %in% unique(eg$group))) {
      stop("selected treatment groups not found in data")
    }
    eg <- eg[eg$group %in% groups, ]
  }
  
  # hclust -> main tree
  gs <- eg$group_id
  gns <- eg$group
  v <- eg$mean
  names(v) <- eg$group_id
  hc <- hclust(dist(v, method = hc_dist), method = hc_link)
  main_ph <- as.phylo(x = hc)
  
  # extract posterior
  e <- extract(x$f, par = "mu_group")$mu_group[, gs]
  e <- e[sample(x = seq_len(nrow(e)), size = min(nrow(e), B),replace = TRUE),]
  
  boot_phs <- lapply(X = seq_len(nrow(e)), FUN = function(i) {
    hc <- hclust(dist(e[i,], method = hc_dist), method = hc_link)
    return(as.phylo(x = hc))
  })
  
  clades <- prop.clades(phy = main_ph, x = c(boot_phs), part = NULL,
                        rooted = is.rooted(main_ph))
  
  main_ph$node.label <- clades
  
  if(all(main_ph$tip.label == gs)) {
    main_ph$tip.label <- gns
  }
  
  # b = 0 for these nodes
  na_nodes <- which(is.na(main_ph$node.label))
  if(length(na_nodes)!=0) {
    main_ph$node.label[na_nodes] <- 0
  }
  
  return(list(main_ph = main_ph, boot_ph = c(boot_phs)))
}

###### testing profiles #######

test_that("profiles functions with invalid inputs", {
  data("d", package = "cellmig")
  d <- d[d$compound %in% c("C1", "C2"),]
  d <- d[d$plate == "1", ]
  o <- suppressWarnings(cellmig(x = d, control = list(mcmc_chains = 1,
                                                      mcmc_warmup = 200,
                                                      mcmc_steps = 500)))
  groups <- get_groups(x = o)$group
  expect_error(get_dose_response_profile(x = o, groups = groups),
               regexp = "must have n >= 2 objects to cluster")
  expect_error(get_dose_response_profile(), regexp = "x is missing or NULL")
  expect_error(get_dose_response_profile(x = o, 
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = NA,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = NULL,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = character(length = 1),
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = c("average", "complete"),
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = NA,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = NULL,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = character(length = 1),
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = c("average", "complete"),
                                         groups = groups))
  

  expect_no_error(get_treatment_profile(x = o, groups = groups))
  expect_error(get_treatment_profile(), regexp = "x is missing or NULL")
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = NA,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = NULL,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = character(length = 1),
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = c("average", "complete"),
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = NA,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = NULL,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = character(length = 1),
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = c("average", "complete"),
                                     groups = groups))
})



###### testing profiles #######

test_that("profiles functions with invalid inputs", {
  data("d", package = "cellmig")
  d <- d[d$compound %in% c("C1", "C2", "C3"),]
  d <- d[d$plate == "1", ]
  o <- suppressWarnings(cellmig(x = d, control = list(mcmc_chains = 1,
                                                      mcmc_warmup = 200,
                                                      mcmc_steps = 500)))
  groups <- get_groups(x = o)$group
  
  
  expect_no_error(get_dose_response_profile(x = o, groups = groups))
  expect_error(get_dose_response_profile(), regexp = "x is missing or NULL")
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = NA,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = NULL,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = character(length = 1),
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_link = c("average", "complete"),
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = NA,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = NULL,
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = character(length = 1),
                                         groups = groups))
  expect_error(get_dose_response_profile(x = o, 
                                         hc_dist = c("average", "complete"),
                                         groups = groups))
  
  
  expect_no_error(get_treatment_profile(x = o, groups = groups))
  expect_error(get_treatment_profile(), regexp = "x is missing or NULL")
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = NA,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = NULL,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = character(length = 1),
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_link = c("average", "complete"),
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = NA,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = NULL,
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = character(length = 1),
                                     groups = groups))
  expect_error(get_treatment_profile(x = o, 
                                     hc_dist = c("average", "complete"),
                                     groups = groups))
})

check_positive_integer <- function(y, par) {
    check_missing(y = y, par = par)
    check_length_one(y = y, par = par)
    check_na_val(y = y, par = par)
    check_numeric_val(y = y, par = par)
    check_inf_val(y = y, par = par)
    if(y <= 0) {
        str <- paste0(par, " must be >0")
        stop(str)
    }
    if(any(!(abs(y - round(y)) < .Machine$double.eps^0.5))) {
        str <- paste0(par, " must be an integer")
        stop(str)
    }
}
check_sd <- function(y, par) {
    check_missing(y = y, par = par)
    check_length_one(y = y, par = par)
    check_numeric_val(y = y, par = par)
    check_na_val(y = y, par = par)
    if(y < 0) {
        str <- paste0(par, " must be >0")
        stop(str)
    }
    check_inf_val(y = y, par = par)
}
check_loc <- function(y, par) {
    check_missing(y = y, par = par)
    check_length_one(y = y, par = par)
    check_na_val(y = y, par = par)
    check_numeric_val(y = y, par = par)
    check_inf_val(y = y, par = par)
}
check_delta <- function(x, o) {
    check_missing(y = x, par = "delta")
    if(length(x)<=1) {
        stop("delta must have length > 1")
    }
    if(any(is.na(x) | is.infinite(x))) {
        stop("some deltas are NA")
    }
    check_numeric_vec(y = x, par = "delta")
    
    # offset
    check_missing(y = o, par = "offset")
    check_length_one(y = o, par = "offset")
    check_na_val(y = o, par = "offset")
    check_numeric_val(y = o, par = "offset")
    if(o <= 0) {
        stop("offset must be greater than 0")
    }
    if(any(!(abs(o - round(o)) < .Machine$double.eps^0.5))) {
        stop("offset must be an integer")
    }
    if(o > length(x)) {
        stop("offset must be between 1 and length of delta")
    }
}
check_generic <- function(y) {
    check_missing(y = y, par = "x")
    check_list(y = y, par = "x")
    if(any(names(y)=="posteriors")==FALSE) {
        stop("wrong y")
    }
}

check_missing <- function(y, par) {
    if(missing(y) || is.null(y)) {
        str <- paste0(par, " is missing or NULL")
        stop(str)
    }
}
check_length_one <- function(y, par) {
    if(length(y)!=1) {
        str <- paste0(par, " must have length of 1")
        stop(str)
    }
}
check_list <- function(y, par) {
    if(is.list(y) == FALSE) {
        str <- paste0(par, " must be list")
        stop(str)
    }
}
check_dataframe <- function(y, par) {
    if(is.data.frame(y) == FALSE) {
        str <- paste0(par, " must be data.frame")
        stop(str)
    }
}
check_numeric_val <- function(y, par) {
    if(is.numeric(y)==FALSE) {
        str <- paste0(par, " must be numeric")
        stop(str)
    }
}
check_numeric_vec <- function(y, par) {
    if(any(is.numeric(y)==FALSE)) {
        str <- paste0(par, " must be numeric")
        stop(str)
    }
}
check_inf_val <- function(y, par) {
    if(is.infinite(y)) {
        str <- paste0(par, " is infinite")
        stop(str)
    }
}
check_na_val <- function(y, par) {
    if(is.na(y)) {
        str <- paste0(par, " is NA")
        stop(str)
    }
}
check_na_vec <- function(y, par) {
    if(any(is.na(y))) {
        str <- paste0(par, " contains NA")
        stop(str)
    }
}
check_logical_val <- function(y, par) {
    if(is.logical(y) == FALSE) {
        str <- paste0(par, " must be logical")
        stop(str)
    }
}
check_character_val <- function(y, par) {
    if(is.character(y) == FALSE) {
        str <- paste0(par, " must be character")
        stop(str)
    }
}
check_character_vec <- function(y, par) {
    if(any(is.character(y)) == FALSE) {
        str <- paste0(par, " must be character")
        stop(str)
    }
}
check_x_in_y <- function(x, y, e) {
    if(!x %in% y) {
        stop(e)
    }
}

get_pmax <- function(x) {
    if(all(x==0)) {
        return(0)
    }
    l <- length(x)
    return(2*max(sum(x<0)/l, sum(x>0)/l)-1)
}

# Description:
# Computes HDI for vector vec and hdi_level (e.g. 0.95)
# Taken (and renamed) from "Doing Bayesian Analysis", section 25.2.3 R code
# for computing HDI of a MCMC sample
get_hdi <- function(vec, hdi_level) {
    sortedPts <- sort(vec)
    ciIdxInc <- floor(hdi_level * length(sortedPts))
    nCIs <- length(sortedPts) - ciIdxInc
    # vapply is preferred for type safety
    ciWidth <- vapply(seq_len(nCIs), function(i) {
        sortedPts[i + ciIdxInc] - sortedPts[i]
    }, numeric(1))
    HDImin <- sortedPts[which.min(ciWidth)]
    HDImax <- sortedPts[which.min(ciWidth) + ciIdxInc]
    HDIlim <- c(HDImin, HDImax)
    return(HDIlim)
}

## ----setup, include = FALSE, warning = FALSE----------------------------------
knitr::opts_chunk$set(comment = FALSE, 
                      message = FALSE)

## -----------------------------------------------------------------------------
library(cellmig)
library(ggplot2)
library(ggforce)
library(patchwork)
library(rstan)

ggplot2::theme_set(new = theme_bw(base_size = 10))

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("cellmig")

## -----------------------------------------------------------------------------
# set seed for reproducible results
set.seed(seed = 1253)
# simulate data
y_p <- gen_partial(control = list(N_biorep = 8, 
                                  N_techrep = 5, 
                                  N_cell = 50, 
                                  delta = c(-0.4, -0.2, -0.1, 0, 0.1, 0.2, 0.4),
                                  sigma_bio = 0.1, 
                                  sigma_tech = 0.05, 
                                  offset = 4,
                                  prior_alpha_p_M = 1.7,
                                  prior_alpha_p_SD = 0.1,
                                  prior_kappa_mu_M = 1.7,
                                  prior_kappa_mu_SD = 0.1,
                                  prior_kappa_sigma_M = 0,
                                  prior_kappa_sigma_SD = 0.1))

## -----------------------------------------------------------------------------
# see data structure
str(y_p$y)

## ----fig.width=7, fig.height=3------------------------------------------------
ggplot(data = y_p$y)+
  geom_sina(aes(x = paste0("t=", group), col = paste0("p=", plate), 
                y = v, group = well), size = 0.5)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  xlab(label = "dose")+
  ylab(label = "migration speed")+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  scale_color_grey(name = "plate")+
  scale_y_log10()

## ----fig.width=7, fig.height=3------------------------------------------------
ggplot(data = aggregate(v~well+group+plate, data = y_p$y, FUN = mean))+
  geom_sina(aes(x = paste0("t=", group), col = paste0("p=", plate), 
                y = v, group = well), size = 1)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  xlab(label = "dose")+
  ylab(label = "migration speed")+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  scale_y_log10()+
  scale_color_grey(name = "plate")

## ----fig.width=7, fig.height=3.5----------------------------------------------
sim_data <- y_p$y

# format simulated data to be used as input in cellmig
sim_data$well <- as.character(sim_data$well)
sim_data$compound <- as.character(sim_data$compound)
sim_data$plate <- as.character(sim_data$plate)
sim_data$offset <- 0
sim_data$offset[sim_data$group=="4"] <- 1

## ----fig.width=7, fig.height=3.5----------------------------------------------
osd <- cellmig(x = sim_data,
               control = list(mcmc_warmup = 250,
                              mcmc_steps = 800,
                              mcmc_chains = 2,
                              mcmc_cores = 2,
                              mcmc_algorithm = "NUTS",
                              adapt_delta = 0.8,
                              max_treedepth = 10))

## ----fig.width=4, fig.height=2------------------------------------------------
ggplot(data = osd$posteriors$alpha_p)+
  geom_point(aes(y = plate_id, x = exp(mean)))+
  geom_errorbarh(aes(y = plate_id, x = exp(mean), xmin = exp(X2.5.), 
                     xmax = exp(X97.5.)), height = 0.1)+
  geom_point(data = data.frame(v = exp(y_p$par$alpha_p),
                               plate = osd$posteriors$alpha_p$plate_id),
             aes(y = plate, x = v), col = "red")+
  scale_y_continuous(breaks = osd$posteriors$alpha_p$plate_id)+
  xlab(label = expression(alpha[p]*"'"))

## ----fig.width=4, fig.height=2.5----------------------------------------------
ggplot(data = osd$posteriors$delta_t)+
  geom_point(aes(y = group_id, x = mean))+
  geom_errorbarh(aes(y = group_id, x = mean, xmin = X2.5., xmax = X97.5.), 
                 height = 0.1)+
  geom_point(data = data.frame(delta = c(-0.4, -0.2, -0.1, 0.1, 0.2, 0.4), 
                               group_id = 1:6),
             aes(y = group_id, x = delta), col = "red")+
  xlab(label = expression(delta[t]))+
  ylab(label = "treatment")+
  scale_y_continuous(breaks = 1:8)

## ----fig.width=4, fig.height=2------------------------------------------------
plot(osd$fit, par = c("sigma_bio", "sigma_tech"))

## -----------------------------------------------------------------------------
control <- list(N_biorep = 4, 
                N_techrep = 3, 
                N_cell = 30, 
                N_group = 8,
                prior_alpha_p_M = 1.7,
                prior_alpha_p_SD = 1.0,
                prior_kappa_mu_M = 1.7,
                prior_kappa_mu_SD = 1.0,
                prior_kappa_sigma_M = 0,
                prior_kappa_sigma_SD = 1.0,
                prior_sigma_bio_M = 0.0,
                prior_sigma_bio_SD = 1.0,
                prior_sigma_tech_M = 0.0,
                prior_sigma_tech_SD = 1.0,
                prior_mu_group_M = 0.0,
                prior_mu_group_SD = 1.0)

## -----------------------------------------------------------------------------
# generate data
y_f <- gen_full(control = control)

## -----------------------------------------------------------------------------
str(y_f)

## ----fig.width=4, fig.height=4------------------------------------------------
w <- data.frame(v_f = y_f$y$v[1:2000], v_p = y_p$y$v[1:2000])
 ggplot(data = w)+
    geom_point(aes(x = v_f, y = v_p), size = 0.5)+
    geom_density_2d(aes(x = v_f, y = v_p), col = "orange")+
    scale_x_log10(name = "Velocity from fully generative model", 
                  limits = c(0.01, 10^4))+
    scale_y_log10(name = "Velocity from partially generative model", 
                  limits = c(0.01, 10^4))+
    annotation_logticks(base = 10, sides = "lb")+
    theme_bw(base_size = 10)

## ----eval = FALSE, message=FALSE, warning=FALSE-------------------------------
# # Constants
# N_bioreps <- c(3, 6, 9)
# N_sim <- 10
# true_deltas <- c(-0.3, -0.15, 0, 0.2, 0.4)
# offset <- 3
# 
# # power analysis
# deltas <- vector(mode = "list", length = length(N_bioreps)*N_sim)
# i <- 1
# for(N_biorep in N_bioreps) {
#   for(b in 1:N_sim) {
#     # simulate data
#     y_p <- gen_partial(control = list(N_biorep = N_biorep,
#                                       N_techrep = 3,
#                                       N_cell = 40,
#                                       delta = true_deltas,
#                                       sigma_bio = 0.1,
#                                       sigma_tech = 0.05,
#                                       offset = offset,
#                                       prior_alpha_p_M = 1.7,
#                                       prior_alpha_p_SD = 0.1,
#                                       prior_kappa_mu_M = 1.7,
#                                       prior_kappa_mu_SD = 0.1,
#                                       prior_kappa_sigma_M = 0,
#                                       prior_kappa_sigma_SD = 0.1))
# 
#     # format simulated data to be used as input in cellmig
#     sim_data <- y_p$y
#     sim_data$well <- as.character(sim_data$well)
#     sim_data$compound <- as.character(sim_data$compound)
#     sim_data$plate <- as.character(sim_data$plate)
#     sim_data$offset <- 0
#     sim_data$offset[sim_data$group==offset] <- 1
# 
#     # run cellmig
#     o <- cellmig(x = sim_data,
#                    control = list(mcmc_warmup = 300,
#                                   mcmc_steps = 1000,
#                                   mcmc_chains = 1,
#                                   mcmc_cores = 1,
#                                   mcmc_algorithm = "NUTS",
#                                   adapt_delta = 0.8,
#                                   max_treedepth = 10))
# 
#     delta <- o$posteriors$delta_t
#     delta$b <- b
#     delta$N_biorep <- N_biorep
#     delta$true_deltas <- true_deltas[-offset]
#     delta$TP <- (delta$X2.5. <= delta$true_deltas &
#                    delta$X97.5. >= delta$true_deltas) &
#       !(delta$X2.5. <= 0 & delta$X97.5. >= 0)
#     deltas[[i]] <- delta
#     i <- i + 1
#   }
# }
# 
# rm(i, delta, o, sim_data, y_p, b, N_biorep, offset, true_deltas)
# 
# deltas <- do.call(rbind, deltas)

## ----eval = FALSE, fig.height=3, fig.width = 6--------------------------------
# ggplot(data = aggregate(TP~N_biorep+true_deltas, data = deltas, FUN = sum))+
#   geom_point(aes(x = N_biorep, y = TP, col = abs(true_deltas),
#                  group = as.factor(true_deltas)), size = 2, alpha = 0.5)+
#   geom_line(aes(x = N_biorep, y = TP, col = abs(true_deltas),
#                 group = as.factor(true_deltas)), alpha = 0.5)+
#   ylab(label = "Number of true positives (TPs)")+
#   scale_color_distiller(name = expression("|"*delta[t]*"|"),palette="Spectral")+
#   theme_bw(base_size = 10)

## -----------------------------------------------------------------------------
sessionInfo()

gen_partial <- function(control = list(N_biorep = 3, 
                                       N_techrep = 3, 
                                       N_cell = 50, 
                                       delta,
                                       sigma_bio = 0.1, 
                                       sigma_tech = 0.05, 
                                       offset = 1,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 0.5,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 0.5,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 0.3)) {
    
    check_sim_control(control = control, partial = TRUE)
    control <- get_sim_control(control_in = control, partial = TRUE)
    o <- get_partial_sim_data(control = control)
    return(list(y = o$yhat, par = o$pars, meta = o$meta, control = control))
}

gen_full <- function(control = list(N_biorep = 3, 
                                    N_techrep = 3, 
                                    N_cell = 50, 
                                    N_group = 5,
                                    prior_alpha_p_M = 1.7,
                                    prior_alpha_p_SD = 1.0,
                                    prior_kappa_mu_M = 1.7,
                                    prior_kappa_mu_SD = 1.0,
                                    prior_kappa_sigma_M = 0,
                                    prior_kappa_sigma_SD = 1.0,
                                    prior_sigma_bio_M = 0.0,
                                    prior_sigma_bio_SD = 1.0,
                                    prior_sigma_tech_M = 0.0,
                                    prior_sigma_tech_SD = 1.0,
                                    prior_mu_group_M = 0.0,
                                    prior_mu_group_SD = 1.0)) {
    
    check_sim_control(control = control, partial = FALSE)
    control <- get_sim_control(control_in = control, partial = FALSE)
    # get meta
    meta <- get_meta_control(control = control)
    
    # sample
    s <- sampling(object = stanmodels$gen_F, algorithm = "Fixed_param", 
                  chains = 1, iter = control$N_cell+10, warmup = 10, 
                  data = control, refresh = -1)
    # extract
    y <- extract(object = s, par = "y")$y
    
    # get data.frame
    y <- melt(data = y)
    colnames(y) <- c("i", "well_id", "v")
    y <- merge(x = y, y = meta, by = "well_id", all.x = TRUE)
    y <- y[y$i <= control$N_cell, ]
    
    y$well <- as.character(y$well_id)
    y$plate <- as.character(y$plate_id)
    y$group <- as.character(y$group_id)
    y$compound <- y$group
    y$dose <- "X"
    y <- y[,c("i", "v", "well", "plate", "group", 
              "compound", "dose", "trep_id")]
    
    return(list(y = y, control = control))
}

check_sim_control <- function(control, partial) {
    check_control_list(control_in = control, partial = partial)
    
    check_positive_integer(y = control$N_biorep, par = "N_biorep")
    check_positive_integer(y = control$N_techrep, par = "N_techrep")
    check_positive_integer(y = control$N_cell, par = "N_cell")
    
    check_sd(y = control$prior_alpha_p_SD, par = "prior_alpha_p_SD")
    check_sd(y = control$prior_kappa_mu_SD, par = "prior_kappa_mu_SD")
    check_sd(y = control$prior_kappa_sigma_SD, par="prior_kappa_sigma_SD")
    
    check_loc(y = control$prior_alpha_p_M, par = "prior_alpha_p_M")
    check_loc(y = control$prior_kappa_mu_M, par = "prior_kappa_mu_M")
    check_sd(y = control$prior_kappa_sigma_M, par = "prior_kappa_sigma_M")
    
    if(partial) {
        check_sd(y = control$sigma_bio, par = "sigma_bio")
        check_sd(y = control$sigma_tech, par = "sigma_tech")
        check_delta(x = control$delta, o = control$offset)
    } 
    else {
        check_sd(y = control$prior_sigma_bio_M, par = "prior_sigma_bio_M")
        check_sd(y = control$prior_sigma_bio_SD, par = "prior_sigma_bio_SD")
        check_sd(y = control$prior_sigma_tech_M, par = "prior_sigma_tech_M")
        check_sd(y = control$prior_sigma_tech_SD, par = "prior_sigma_tech_SD")
        check_sd(y = control$prior_mu_group_SD, par = "prior_mu_group_SD")
        check_sd(y = control$prior_mu_group_M, par = "prior_mu_group_M")
        check_positive_integer(y = control$N_group, par = "N_group")
    }
}

get_sim_control <- function(control_in, partial) {
    if(partial) {
        control <- get_default_control_partial_sim()
        if(all(names(control_in) %in% names(control)) == FALSE) {
            stop("unrecognized elements found in control")
        }
        control[names(control_in)] <- control_in
        control$N_group <- length(control$delta)
        control$N_plate <- control$N_biorep
        control$N_well_reps <- control$N_techrep
        control$mu_group <- control$delta
    } 
    else {
        control <- get_default_control_full_sim()
        if(all(names(control_in) %in% names(control)) == FALSE) {
            stop("unrecognized elements found in control")
        }
        control[names(control_in)] <- control_in
        control$N_plate <- control$N_biorep
        control$N_well_reps <- control$N_techrep
    }
    return(control)
}

get_partial_sim_data <- function(control) {
    control$N_group <- length(control$delta)
    control$N_well <- control$N_biorep*control$N_group*control$N_techrep
    
    well_grid <- expand.grid(g = seq_len(control$N_group),
                             p = seq_len(control$N_biorep),
                             w = seq_len(control$N_techrep))
    mu_plate_group <- t(vapply(X = seq_len(control$N_group), 
                               FUN = function(g) {
                                   return(rnorm(n = control$N_biorep, 
                                                mean = control$delta[g], 
                                                sd = control$sigma_bio))
                               }, FUN.VALUE = numeric(control$N_biorep)))
    
    kappa_mu <- rnorm(n = 1, mean = control$prior_kappa_mu_M, 
                      sd = control$prior_kappa_mu_SD)
    kappa_sigma <- abs(rnorm(n = 1, mean = control$prior_kappa_sigma_M, 
                             sd = control$prior_kappa_sigma_SD))
    alpha_p <- rnorm(n = control$N_biorep, mean = control$prior_alpha_p_M, 
                     sd = control$prior_alpha_p_SD)
    
    
    # Use mapply to compute for each well
    results <- mapply(function(g, p, w, well_id) {
        if(g == control$offset) {
            mu_well <- rnorm(n = 1, mean = alpha_p[p], sd = control$sigma_tech)
        } else {
            mu_well <- rnorm(n = 1, mean = alpha_p[p] + mu_plate_group[g, p], 
                             sd = control$sigma_tech)
        }
        kappa <- exp(rnorm(n = 1, mean = kappa_mu, sd = kappa_sigma))
        mu <- exp(mu_well)
        y <- rgamma(n = control$N_cell, shape = kappa, rate = kappa/mu)
        list(yhat = data.frame(v = y, well = well_id, plate = p, group = g, 
                               compound = g, dose = "X"),
             meta = data.frame(w = well_id, g = g, p = p),
             mu_well = mu_well,
             kappa = kappa,
             mu = mu)
    }, well_grid$g, well_grid$p, well_grid$w, seq_len(nrow(well_grid)), 
    SIMPLIFY = FALSE)
    
    yhat <- do.call(rbind, lapply(results, `[[`, "yhat"))
    meta <- do.call(rbind, lapply(results, `[[`, "meta"))
    mu_well <- vapply(X = results, `[[`, "mu_well", FUN.VALUE = numeric(1))
    kappa <- vapply(X = results, `[[`, "kappa", FUN.VALUE = numeric(1))
    mu <- vapply(X = results, `[[`, "mu", FUN.VALUE = numeric(1))
    
    pars <- list(alpha_p = alpha_p, mu_plate_group = mu_plate_group, 
                 mu = mu, mu_well = mu_well, kappa = kappa)
    return(list(yhat = yhat, pars = pars, meta = meta))
}

get_meta_control <- function(control) {
    meta <- expand.grid(group_id = seq_len(control$N_group),
                        plate_id = seq_len(control$N_plate),
                        trep_id  = seq_len(control$N_well_reps))
    meta$well_id <- seq_len(nrow(meta))
    meta <- meta[, c("well_id", "group_id", "plate_id", "trep_id")]
    return(meta)
}

check_control_list <- function(control_in, partial) {
    control <- get_default_control_full_sim()
    if(partial) {
        control <- get_default_control_partial_sim()
    }
    
    # if missing control_in -> use default values
    check_missing(y = control_in, par = "control")
    check_list(y = control_in, par = "control")
    if(all(names(control_in) %in% names(control))==FALSE) {
        stop("control has missing elements")
    }
}

get_default_control_partial_sim <- function() {
    control <- list(N_biorep = 3, N_techrep = 3, N_cell = 50, 
                    delta = c(0, -0.4, -0.2, -0.1, 0, 0.1, 0.2, 0.4),
                    sigma_bio = 0.2, sigma_tech = 0.05, offset = 1,
                    prior_alpha_p_M = 1.7, prior_alpha_p_SD = 0.5,
                    prior_kappa_mu_M = 1.7, prior_kappa_mu_SD = 0.5,
                    prior_kappa_sigma_M = 0, prior_kappa_sigma_SD = 0.3,
                    prior_sigma_bio_M = 0.0,
                    prior_sigma_bio_SD = 0.0,
                    prior_sigma_tech_M = 0.0,
                    prior_sigma_tech_SD = 0.0,
                    prior_mu_group_M = 0.0,
                    prior_mu_group_SD = 0.0)
    return(control)
}

get_default_control_full_sim <- function() {
    control <- list(N_biorep = 3, N_techrep = 3, N_cell = 50, 
                    N_group = 5,
                    prior_alpha_p_M = 1.7, prior_alpha_p_SD = 1.0,
                    prior_kappa_mu_M = 1.7, prior_kappa_mu_SD = 1.0,
                    prior_kappa_sigma_M = 0, prior_kappa_sigma_SD = 1.0,
                    prior_sigma_bio_M = 0.0, prior_sigma_bio_SD = 1.0,
                    prior_sigma_tech_M = 0.0, prior_sigma_tech_SD = 1.0,
                    prior_mu_group_M = 0.0, prior_mu_group_SD = 1.0,
                    offset = 1)
    return(control)
}

###### testing simulations #######

test_that("get_partial with invalid control", {
  expect_error(gen_partial(),
               regexp = "object 'delta' not found")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta,
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "object 'delta' not found")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = 0,
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "delta must have length > 1")
  expect_no_error(gen_partial(control = list(N_biorep = 3, 
                                             N_techrep = 3, 
                                             N_cell = 50, 
                                             delta = c(0,2),
                                             sigma_bio = 0.2, 
                                             sigma_tech = 0.05, 
                                             offset = 1,
                                             prior_alpha_p_M = 1.7,
                                             prior_alpha_p_SD = 0.5,
                                             prior_kappa_mu_M = 1.7,
                                             prior_kappa_mu_SD = 0.5,
                                             prior_kappa_sigma_M = 0,
                                             prior_kappa_sigma_SD = 0.3)))
  
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 3,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "offset must be between 1 and length of delta")
  
  expect_error(gen_partial(control = NA),
               regexp = "control must be list")
  expect_error(gen_partial(control = NULL),
               regexp = "control is missing or NULL")
  expect_error(gen_partial(control = NA),
               regexp = "control must be list")
  
  
  
  expect_no_error(gen_partial(control = list(N_biorep = 3, 
                                             N_techrep = 3, 
                                             N_cell = 50, 
                                             delta = c(0,2),
                                             sigma_bio = 0.2, 
                                             sigma_tech = 0.05, 
                                             offset = 1,
                                             prior_alpha_p_M = -1.7,
                                             prior_alpha_p_SD = 0.5,
                                             prior_kappa_mu_M = 1.7,
                                             prior_kappa_mu_SD = 0.5,
                                             prior_kappa_sigma_M = 0,
                                             prior_kappa_sigma_SD = 0.3)))
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = -0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_alpha_p_SD must be >0")
  expect_no_error(gen_partial(control = list(N_biorep = 3, 
                                             N_techrep = 3, 
                                             N_cell = 50, 
                                             delta = c(0,2),
                                             sigma_bio = 0.2, 
                                             sigma_tech = 0.05, 
                                             offset = 1,
                                             prior_alpha_p_M = 1.7,
                                             prior_alpha_p_SD = 0,
                                             prior_kappa_mu_M = 1.7,
                                             prior_kappa_mu_SD = 0.5,
                                             prior_kappa_sigma_M = 0,
                                             prior_kappa_sigma_SD = 0.3)))
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = -Inf,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_alpha_p_SD must be >0")
  
  
  
  expect_no_error(gen_partial(control = list(N_biorep = 3, 
                                             N_techrep = 3, 
                                             N_cell = 50, 
                                             delta = c(0,2),
                                             sigma_bio = 0.2, 
                                             sigma_tech = 0.05, 
                                             offset = 1,
                                             prior_alpha_p_M = 1.7,
                                             prior_alpha_p_SD = 0.5,
                                             prior_kappa_mu_M = -1.7,
                                             prior_kappa_mu_SD = 0.5,
                                             prior_kappa_sigma_M = 0,
                                             prior_kappa_sigma_SD = 0.3)))
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = -0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_kappa_mu_SD must be >0")
  expect_no_error(gen_partial(control = list(N_biorep = 3, 
                                             N_techrep = 3, 
                                             N_cell = 50, 
                                             delta = c(0,2),
                                             sigma_bio = 0.2, 
                                             sigma_tech = 0.05, 
                                             offset = 1,
                                             prior_alpha_p_M = 1.7,
                                             prior_alpha_p_SD = 0.5,
                                             prior_kappa_mu_M = 1.7,
                                             prior_kappa_mu_SD = 0,
                                             prior_kappa_sigma_M = 0,
                                             prior_kappa_sigma_SD = 0.3)))
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = -Inf,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_kappa_mu_SD must be >0")
  
  
  expect_no_error(gen_partial(control = list(N_biorep = 3, 
                                             N_techrep = 3, 
                                             N_cell = 50, 
                                             delta = c(0,2),
                                             sigma_bio = 0.2, 
                                             sigma_tech = 0.05, 
                                             offset = 1,
                                             prior_alpha_p_M = 1.7,
                                             prior_alpha_p_SD = 0.5,
                                             prior_kappa_mu_M = .7,
                                             prior_kappa_mu_SD = 0.5,
                                             prior_kappa_sigma_M = 0,
                                             prior_kappa_sigma_SD = 0.3)))
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = -1,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_kappa_sigma_M must be >0")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = Inf,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_kappa_sigma_M is infinite")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = -Inf,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_kappa_sigma_M must be >0")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 1.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = NA,
                                          prior_kappa_sigma_SD = 0.3)),
               regexp = "prior_kappa_sigma_M must be numeric")
  
  
  expect_no_error(gen_partial(control = list(N_biorep = 3, 
                                             N_techrep = 3, 
                                             N_cell = 50, 
                                             delta = c(0,2),
                                             sigma_bio = 0.2, 
                                             sigma_tech = 0.05, 
                                             offset = 1,
                                             prior_alpha_p_M = 1.7,
                                             prior_alpha_p_SD = 0.5,
                                             prior_kappa_mu_M = 0.7,
                                             prior_kappa_mu_SD = 0.5,
                                             prior_kappa_sigma_M = 0,
                                             prior_kappa_sigma_SD = 0.3)))
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 0.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = -0.3)),
               regexp = "prior_kappa_sigma_SD must be >0")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 0.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = -Inf)),
               regexp = "prior_kappa_sigma_SD must be >0")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 0.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = Inf)),
               regexp = "prior_kappa_sigma_SD is infinite")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 0.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = NA)),
               regexp = "prior_kappa_sigma_SD must be numeric")
  
  
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = 1.7,
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 0.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = c(0, 1))),
               regexp = "prior_kappa_sigma_SD must have length of 1")
  expect_error(gen_partial(control = list(N_biorep = 3, 
                                          N_techrep = 3, 
                                          N_cell = 50, 
                                          delta = c(0,2),
                                          sigma_bio = 0.2, 
                                          sigma_tech = 0.05, 
                                          offset = 1,
                                          prior_alpha_p_M = c(1.7, 2),
                                          prior_alpha_p_SD = 0.5,
                                          prior_kappa_mu_M = 0.7,
                                          prior_kappa_mu_SD = 0.5,
                                          prior_kappa_sigma_M = 0,
                                          prior_kappa_sigma_SD = 0)),
               regexp = "prior_alpha_p_M must have length of 1")
})

test_that("get_full with simple controls", {
  expect_no_error(gen_full())
  expect_error(gen_full(control = NA),
               regexp = "control must be list")
  expect_error(gen_full(control = NULL),
               regexp = "control is missing or NULL")
  expect_error(gen_full(control = NA),
               regexp = "control must be list")
})

test_that("get_full with wrong control entries", {
  expect_error(gen_full(control = list(N_biorep = NA, 
                                       N_techrep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_biorep is NA")
  expect_error(gen_full(control = list(N_biorep = NULL, 
                                       N_techrep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_biorep is missing or NULL")
  expect_error(gen_full(control = list(N_techrep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_biorep is missing or NULL")
  expect_error(gen_full(control = list(N_biorep = -1, 
                                       N_techrep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_biorep must be >0")
  expect_error(gen_full(control = list(N_biorep = 1.5, 
                                       N_techrep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_biorep must be an integer")
  expect_error(gen_full(control = list(N_biorep = Inf, 
                                       N_techrep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_biorep is infinite")
  
  
  
  expect_error(gen_full(control = list(N_techrep = NA, 
                                       N_biorep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_techrep is NA")
  expect_error(gen_full(control = list(N_techrep = NULL, 
                                       N_biorep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_techrep is missing or NULL")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_techrep is missing or NULL")
  expect_error(gen_full(control = list(N_techrep = -1, 
                                       N_biorep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_techrep must be >0")
  expect_error(gen_full(control = list(N_techrep = 1.5, 
                                       N_biorep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_techrep must be an integer")
  expect_error(gen_full(control = list(N_techrep = Inf, 
                                       N_biorep = 3,
                                       N_cell = 50,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_techrep is infinite")
  
  
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_techrep = 3,
                                       N_cell = NA, 
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_cell is NA")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_techrep = 3,
                                       N_cell = NULL,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_cell is missing or NULL")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_techrep = 3,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_cell is missing or NULL")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_techrep = 3,
                                       N_cell = -1,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_cell must be >0")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_techrep = 3,
                                       N_cell = 1.5,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_cell must be an integer")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_techrep = 3,
                                       N_cell = Inf,
                                       N_group = 5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "N_cell is infinite")
  
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_tech = 3,
                                       N_cell = 10, 
                                       N_group = NA,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "control has missing elements")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_tech = 3,
                                       N_cell = 10,
                                       N_group = NULL,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "control has missing elements")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_tech = 3,
                                       N_cell = 10,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "control has missing elements")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_tech = 3,
                                       N_cell = 10,
                                       N_group = -1,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "control has missing elements")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_tech = 3,
                                       N_cell = 10,
                                       N_group = 1.5,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "control has missing elements")
  expect_error(gen_full(control = list(N_biorep = 3,
                                       N_tech = 3,
                                       N_cell = 10,
                                       N_group = Inf,
                                       prior_alpha_p_M = 1.7,
                                       prior_alpha_p_SD = 1,
                                       prior_kappa_mu_M = 1.7,
                                       prior_kappa_mu_SD = 1,
                                       prior_kappa_sigma_M = 0,
                                       prior_kappa_sigma_SD = 1,
                                       prior_sigma_bio_M = 0,
                                       prior_sigma_bio_SD = 1,
                                       prior_sigma_tech_M = 0,
                                       prior_sigma_tech_SD = 1,
                                       prior_mu_group_M = 0,
                                       prior_mu_group_SD = 1)),
               regexp = "control has missing elements")
})

# Generated by rstantools.  Do not edit by hand.

# names of stan models
stanmodels <- c("M", "gen_F")

# load each stan module
Rcpp::loadModule("stan_fit4M_mod", what = TRUE)
Rcpp::loadModule("stan_fit4gen_F_mod", what = TRUE)

# instantiate each stanmodel object
stanmodels <- sapply(stanmodels, function(model_name) {
  # create C++ code for stan model
  stan_file <- if(dir.exists("stan")) "stan" else file.path("inst", "stan")
  stan_file <- file.path(stan_file, paste0(model_name, ".stan"))
  stanfit <- rstan::stanc_builder(stan_file,
                                  allow_undefined = TRUE,
                                  obfuscate_model_name = FALSE)
  stanfit$model_cpp <- list(model_cppname = stanfit$model_name,
                            model_cppcode = stanfit$cppcode)
  # create stanmodel object
  methods::new(Class = "stanmodel",
               model_name = stanfit$model_name,
               model_code = stanfit$model_code,
               model_cpp = stanfit$model_cpp,
               mk_cppmodule = function(x) get(paste0("rstantools_model_", model_name)))
})

## ----setup, include = FALSE, warning = FALSE----------------------------------
knitr::opts_chunk$set(comment = FALSE, 
                      warning = FALSE, 
                      message = FALSE)

## -----------------------------------------------------------------------------
library(cellmig)
library(ggplot2)
library(ggforce)
ggplot2::theme_set(new = theme_bw(base_size = 10))

## ----eval=FALSE---------------------------------------------------------------
# if (!require("BiocManager", quietly = TRUE))
#     install.packages("BiocManager")
# 
# BiocManager::install("cellmig")

## -----------------------------------------------------------------------------
data("d", package = "cellmig")
str(d)

## ----fig.width=7, fig.height=6------------------------------------------------
ggplot(data = d)+
  facet_wrap(facets = ~paste0("compound=", compound), 
             scales = "free_y", ncol = 2)+
  geom_sina(aes(x = as.factor(dose), col = plate, y = v, group = well), 
            size = 0.5)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  ylab(label = "migration velocity")+
  xlab(label = '')+
  scale_color_grey()+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  guides(shape = guide_legend(override.aes = list(size = 3)))+
  scale_y_log10()+
  annotation_logticks(base = 10, sides = "l")

## ----fig.width=7, fig.height=6------------------------------------------------
dm <- aggregate(v~well+plate+compound+dose, data = d, FUN = mean)
ggplot(data = dm)+
  facet_wrap(facets = ~paste0("compound=", compound), 
             scales = "free_y", ncol = 2)+
  geom_sina(aes(x = as.factor(dose), col = plate, y = v, group = well), 
            size = 1.5, alpha = 0.7)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  ylab(label = "migration velocity")+
  xlab(label = '')+
  scale_color_grey()+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  guides(shape = guide_legend(override.aes = list(size = 3)))+
  scale_y_log10()+
  annotation_logticks(base = 10, sides = "l")

## ----fig.width=7, fig.height=3.5----------------------------------------------
o <- cellmig(x = d,
             control = list(mcmc_warmup = 300, # nr. of MCMC warmup step?
                            mcmc_steps = 1000, # nr. of MCMC iteration steps?
                            mcmc_chains = 2,   # nr. of MCMC chains
                            mcmc_cores = 2))   # nr. of MCMC cores

## -----------------------------------------------------------------------------
str(o$posteriors$delta_t)

## ----fig.width=6, fig.height=3.3----------------------------------------------
ggplot(data = o$posteriors$delta_t)+
  geom_line(aes(x = dose, y = mean, col = compound, group = compound))+
  geom_point(aes(x = dose, y = mean, col = compound))+
  geom_errorbar(aes(x = dose, y = mean, ymin = X2.5., ymax = X97.5., 
                    col = compound), width = 0.1)+
  ylab(label = expression("Overall treatment effect ("*delta*")"))+
  theme(legend.position = "top")

## ----fig.width=9.5, fig.height=5----------------------------------------------
get_dose_response_profile(x = o)+
  patchwork::plot_layout(widths = c(.7, 1, 4))

## ----fig.width=14, fig.height=6-----------------------------------------------
u <- get_pairs(x = o, exponentiate = FALSE)
u$plot

## ----fig.width=7, fig.height=2.5----------------------------------------------
get_groups(x = o)
u <- get_violins(x = o, 
                 from_groups = get_groups(x = o)$group,
                 to_group = "C2|D1",
                 exponentiate = FALSE)
u$plot

## ----fig.width=5, fig.height=6------------------------------------------------
g <- get_ppc_violins(x = o, wrap = TRUE, ncol = 3)
g+scale_y_log10()

## ----fig.width=4, fig.height=4------------------------------------------------
g <- get_ppc_means(x = o)
g

## ----fig.height=2, fig.width=6------------------------------------------------
g_alpha_p <- ggplot(data = o$posteriors$alpha_p)+
  geom_errorbarh(aes(y = plate_id, x = mean, xmin = X2.5., xmax = X97.5.),
                 height = 0.1)+
  geom_point(aes(y = plate_id, x = mean))

g_sigma <- ggplot()+
  geom_errorbarh(data = o$posteriors$sigma_bio,
                 aes(y = "sigma_bio",
                     x = mean, xmin = X2.5., xmax = X97.5.),
                 height = 0.1)+
  geom_errorbarh(data = o$posteriors$sigma_tech,
                 aes(y = "sigma_tech",
                     x = mean, xmin = X2.5., xmax = X97.5.),
                 height = 0.1)+
  geom_point(data = o$posteriors$sigma_bio,
             aes(y = "sigma_bio", x = mean))+
  geom_point(data = o$posteriors$sigma_tech,
             aes(y = "sigma_tech", x = mean))+
  ylab(label = '')

g_alpha_p|g_sigma

## -----------------------------------------------------------------------------
sessionInfo()

process_input <- function(x) {
  
  check_x <- function(x) {
    check_missing(y = x, par = "x")
    check_dataframe(y = x, par = "x")
    if(nrow(x)<=1) {
      stop("zero or one row in x")
    }
  }
  
  check_x(x=x)
  check_input_columns(x=x)
  
  if(any(x$offset == 1)) {
    return(get_input_with_offset(x = x))
  } else {
    return(get_input_without_offset(x = x))
  }
}

process_control <- function(control_in) {
  control <- get_default_control()
  
  # if missing control_in -> use default values
  check_missing(y = control_in, par = "control_in")
  check_list(y = control_in, par = "control_in")
  
  if(all(names(control_in) %in% names(control)) == FALSE) {
    stop("unrecognized elements found in control")
  }
  
  ns <- names(control_in)
  control[names(control_in)] <- control_in
  
  check_positive_integer(y = control$mcmc_chains, par = "mcmc_chains")
  check_positive_integer(y = control$mcmc_cores, par = "mcmc_cores")
  check_positive_integer(y = control$mcmc_steps, par = "mcmc_steps")
  check_positive_integer(y = control$mcmc_warmup, par = "mcmc_warmup")
  
  check_mcmc_steps(mcmc_steps = control$mcmc_steps, 
                   mcmc_warmup = control$mcmc_warmup)
  
  return(control)
}

# MCMC steps vs. warmup
check_mcmc_steps <- function(mcmc_steps, mcmc_warmup) {
  if(as.integer(x = mcmc_steps) < 500 |
     as.integer(x = mcmc_warmup) < 100) {
    stop("mcmc_steps >= 500 & mcmc_warmup >= 100.")
  }
  
  if(as.integer(x = mcmc_steps) <= as.integer(x = mcmc_warmup)) {
    stop("mcmc_steps > mcmc_warmup")
  }
}

get_default_control <- function() {
  return(list(mcmc_warmup = 500,
              mcmc_steps = 1500,
              mcmc_chains = 4,
              mcmc_cores = 1,
              mcmc_algorithm = "NUTS",
              adapt_delta = 0.8,
              max_treedepth = 10,
              prior_alpha_p_M = 1.7,
              prior_alpha_p_SD = 1.0,
              prior_sigma_bio_M = 0.0,
              prior_sigma_bio_SD = 1.0,
              prior_sigma_tech_M = 0.0,
              prior_sigma_tech_SD = 1.0,
              prior_kappa_mu_M = 1.7,
              prior_kappa_mu_SD = 1.0,
              prior_kappa_sigma_M = 0.0,
              prior_kappa_sigma_SD = 1.0,
              prior_mu_group_M = 0.0,
              prior_mu_group_SD = 1.0))
}

check_input_columns <- function(x) {
  check_x_in_y(x = "compound", y = colnames(x), e = "compund not found in x")
  check_x_in_y(x = "dose", y = colnames(x), e = "dose not found in x")
  check_x_in_y(x = "well", y = colnames(x), e = "well not found in x")
  check_x_in_y(x = "plate", y = colnames(x), e = "plate not found in x")
  check_x_in_y(x = "v", y = colnames(x), e = "v not found in x")
  check_x_in_y(x = "offset", y = colnames(x), e = "offset not found in x")
  
  check_character_vec(y = x[,"compound"], par = "compound")
  check_character_vec(y = x[,"dose"], par = "dose")
  check_character_vec(y = x[,"well"], par = "well")
  check_character_vec(y = x[,"plate"], par = "plate")
  check_numeric_vec(y = x[,"v"], par = "v")
  check_numeric_vec(y = x[,"offset"], par = "offset")
  
  check_na_vec(y = x[,"compound"], par = "compound")
  if(length(unique(x$compound))==1) {
    stop("only one compound included")
  }
  check_na_vec(y = x[,"dose"], par = "dose")
  check_na_vec(y = x[,"plate"], par = "plate")
  check_na_vec(y = x[,"well"], par = "well")
  check_na_vec(y = x[,"v"], par = "v")
  check_na_vec(y = x[,"offset"], par = "offset")
  
  if(all(x[,"offset"] %in% c(0,1))==FALSE) {
    stop("column offset can contain only 0 or 1")
  }
}

get_input_with_offset <- function(x) {
  org_x <- x
  
  x$plate_id <- as.numeric(as.factor(x$plate))
  x$group <- paste0(x$compound, '|', x$dose)
  x$well <- paste0(x$plate, '|', x$well, '|', x$group)
  x$plate_group <- paste0(x$plate, '|', x$group)
  x$well_id <- as.numeric(as.factor(x$well))
  
  xs <- x[x$offset == 0,]
  xr <- x[x$offset == 1,]
  
  xs$group_id <- as.numeric(as.factor(xs$group))
  xr$group_id <- 0
  
  xs$plate_group_id <- as.numeric(as.factor(xs$plate_group))
  xr$plate_group_id <- 0
  
  x <- rbind(xr, xs)
  
  x$N <- nrow(x)
  x$N_well <- max(x$well_id)
  x$N_plate <- max(x$plate_id)
  x$N_plate_group <- max(x$plate_group_id, na.rm = TRUE)
  x$N_group <- max(x$group_id)
  
  q <- x[, c("well_id", "plate_id", "plate_group_id", "offset")]
  q <- q[duplicated(q) == FALSE, ]
  q <- q[order(q$well_id, decreasing = FALSE),]
  
  # group_id -> plate_group_id 
  z <- x[, c("plate_group_id", "group_id")]
  z <- z[duplicated(z) == FALSE, ]
  z <- z[order(z$plate_group_id, decreasing = FALSE),]
  z <- z[z$group_id !=0,]
  
  return(list(proc_x = x, org_x = org_x, map_w = q, map_pg = z))
}

get_input_without_offset <- function(x) {
  org_x <- x
  
  x$plate_id <- as.numeric(as.factor(x$plate))
  
  x$group <- paste0(x$compound, '|', x$dose)
  x$group_id <- as.numeric(as.factor(x$group))
  
  x$well <- paste0(x$plate, '|', x$well, '|', x$group)
  x$well_id <- as.numeric(as.factor(x$well))
  
  x$plate_group <- paste0(x$plate, '|', x$group)
  x$plate_group_id <- as.numeric(as.factor(x$plate_group))
  
  x$N <- nrow(x)
  x$N_well <- max(x$well_id)
  x$N_plate <- max(x$plate_id)
  x$N_plate_group <- max(x$plate_group_id, na.rm = TRUE)
  x$N_group <- max(x$group_id)
  
  q <- x[, c("well_id", "plate_id", "plate_group_id", "offset")]
  q <- q[duplicated(q) == FALSE, ]
  q <- q[order(q$well_id, decreasing = FALSE),]
  
  # group_id -> plate_group_id 
  z <- x[, c("plate_group_id", "group_id")]
  z <- z[duplicated(z) == FALSE, ]
  z <- z[order(z$plate_group_id, decreasing = FALSE),]
  
  return(list(proc_x = x, org_x = org_x, map_w = q, map_pg = z))
}

get_ppc_violins <- function(x, wrap = FALSE, ncol = 4) {
    
    check_missing(y = x, par = "x")
    check_missing(y = wrap, par = "wrap")
    check_length_one(y = wrap, par = "wrap")
    check_logical_val(y = wrap, par = "wrap")
    check_na_val(y = wrap, par = "wrap")

    if(wrap) {
        check_missing(y = ncol, par = "ncol")
        check_length_one(y = ncol, par = "ncol")
        check_numeric_val(y = ncol, par = "ncol")
        check_positive_integer(y = ncol, par = "ncol")
        check_na_val(y = ncol, par = "ncol")
        check_inf_val(y = ncol, par = "ncol")
    }
    
    e <- extract(object = x$fit, par = "y_hat_sample")$y_hat_sample
    e <- melt(data = e)
    colnames(e) <- c("iter", "well_id", "yhat")
    q <- x$data$proc_x[, c("well_id", "compound", "dose", 
                           "group", "plate", "well")]
    q <- q[duplicated(q)==FALSE,]
    e <- merge(x = e, y = q, all.x = TRUE)
    
    if(wrap==FALSE) {
        g <- ggplot()+
            facet_grid(compound~plate, scales = "free")+
            geom_sina(data = x$data$proc_x, aes(x = as.factor(dose), 
                                        y = v, group = well), 
                      col = "black", size = 0.3)+
            geom_violin(data = e, aes(x = as.factor(dose), 
                                      y = yhat, group = well), 
                        fill = NA, col = "#f75ea3", alpha = 0.35)+
            theme_bw(base_size = 10)+
            theme(legend.position = "none")+
            theme(strip.text.x = element_text(
                margin = margin(0.02,0,0.02,0, "cm")))+
            xlab(label = "dose")+
            scale_y_continuous(name = "Cell migration speed", 
                               breaks = pretty_breaks(3))
    }
    
    if(wrap) {
        g <- ggplot()+
            facet_wrap(~compound+plate, scales = "free", ncol = ncol)+
            geom_sina(data = x$data$proc_x, aes(x = as.factor(dose), 
                                        y = v, group = well), 
                      col = "black", size = 0.3)+
            geom_violin(data = e, aes(x = as.factor(dose), 
                                      y = yhat, group = well), 
                        fill = NA, col = "#f75ea3", alpha = 0.35)+
            theme_bw(base_size = 10)+
            theme(legend.position = "none")+
            theme(strip.text.x = element_text(
                margin = margin(0.02,0,0.02,0, "cm")))+
            xlab(label = "dose")+
            scale_y_continuous(name = "Cell migration speed", 
                               breaks = pretty_breaks(3))
    }
    
    return(g)
}

get_ppc_means <- function(x) {
    check_missing(y = x, par = "x")
    
    y <- aggregate(v~well_id, data = x$data$proc_x, FUN = mean)
    yhat <- x$posteriors$yhat
    y <- merge(x = y, y = yhat, by = "well_id")
    
    g <- ggplot(data = y)+
        geom_abline(slope = 1, intercept = 0, linetype = "dashed")+
        geom_errorbar(aes(x = v, y = mean, ymin = X2.5., ymax = X97.5.), 
                      col = "darkgray")+
        geom_point(aes(x = v, y = mean), shape = 21, 
                   fill = "white", alpha = 0.75)+
        theme_bw(base_size = 10)+
        xlab(label = "Observed migration + 95% HDI")+
        ylab(label = "Predicted migration")
    
    return(g)
}

###### testing others #######

test_that("test ppc.R error", {
  expect_error(get_ppc_means(), 
               regexp = "x is missing or NULL")
  expect_error(get_ppc_means(x = NULL), 
               regexp = "x is missing or NULL")
  expect_error(get_ppc_violins(), 
               regexp = "x is missing or NULL")
  expect_error(get_ppc_violins(x = NULL), 
               regexp = "x is missing or NULL")
})


test_that("test ppc.R correct", {
  data("d", package = "cellmig")
  d <- d[d$compound %in% c("C1", "C2"),]
  d <- d[d$plate == "1", ]
  o <- suppressWarnings(cellmig(x = d, control = list(mcmc_chains = 1,
                                                      mcmc_warmup = 200,
                                                      mcmc_steps = 500)))
  
  expect_no_error(get_ppc_means(x = o))
  expect_no_error(get_ppc_violins(x = o, wrap = FALSE))
  
  expect_no_error(get_ppc_violins(x = o, wrap = TRUE))
  expect_error(get_ppc_violins(x = o, wrap = NULL), 
               regexp = "wrap is missing or NULL")
  expect_no_error(get_ppc_violins(x = o, ncol = NULL))
  expect_error(get_ppc_violins(x = o, wrap = TRUE, ncol = -1),
               regexp = "ncol must be >0")
  expect_error(get_ppc_violins(x = o, wrap = TRUE, ncol = TRUE),
               regexp = "ncol must be numeric")
  expect_error(get_ppc_violins(x = o, wrap = TRUE, ncol = NA),
               regexp = "ncol must be numeric")
  expect_error(get_ppc_violins(x = o, wrap = TRUE, ncol = NULL),
               regexp = "ncol is missing or NULL")
  
  
  expect_no_error(get_ppc_violins(x = o, wrap = TRUE, ncol = 1))
})

cellmig <- function(x, control = NULL) {
  
  # check inputs
  x <- process_input(x)
  
  # check control
  control <- process_control(control_in = control)
  
  # fit model
  f <- get_fit(x = x, control = control)
  
  # get summary
  s <- get_summary(x = x, f = f)
  
  return(list(fit = f, posteriors = s, data = x, control = control))
}

get_fit <- function(x, control) {
  message("model fitting... \n")
  
  M <- stanmodels$M
  
  pars <- c("z_1", "z_2", "z_3", "mu_well")
  
  # fit model
  fit <- sampling(object=M,
                  data=list(y=x$proc_x$v, 
                            N=x$proc_x$N[1], 
                            N_well=x$proc_x$N_well[1], 
                            N_plate=x$proc_x$N_plate[1],
                            N_plate_group=x$proc_x$N_plate_group[1],
                            N_group=x$proc_x$N_group[1],
                            well_id=x$proc_x$well_id,
                            plate_id=x$map_w$plate_id,
                            plate_group_id=x$map_w$plate_group_id,
                            offset=x$map_w$offset,
                            group_id=x$map_pg$group_id,
                            prior_alpha_p_M=control$prior_alpha_p_M,
                            prior_alpha_p_SD=control$prior_alpha_p_SD,
                            prior_sigma_bio_M=control$prior_sigma_bio_M,
                            prior_sigma_bio_SD=control$prior_sigma_bio_SD,
                            prior_sigma_tech_M=control$prior_sigma_tech_M,
                            prior_sigma_tech_SD=control$prior_sigma_tech_SD,
                            prior_kappa_mu_M=control$prior_kappa_mu_M,
                            prior_kappa_mu_SD=control$prior_kappa_mu_SD,
                            prior_kappa_sigma_M=control$prior_kappa_sigma_M,
                            prior_kappa_sigma_SD=control$prior_kappa_sigma_SD,
                            prior_mu_group_M=control$prior_mu_group_M,
                            prior_mu_group_SD=control$prior_mu_group_SD),
                  pars=pars,
                  include = FALSE,
                  chains=control$mcmc_chains,
                  cores=control$mcmc_cores,
                  iter=control$mcmc_steps,
                  warmup=control$mcmc_warmup,
                  algorithm=control$mcmc_algorithm,
                  control=list(adapt_delta=control$adapt_delta,
                               max_treedepth=control$max_treedepth),
                  refresh=200)
  
  return(fit)
}

get_summary <- function(x, f) {
  message("computing posterior summaries...\n")
  
  x <- x$proc_x
  
  # get meta data
  l <- x[, c("well_id", "well", "group_id", "group", 
             "compound", "dose", "plate_id", "plate", 
             "plate_group_id", "plate_group", "offset")]
  meta_well <- l[duplicated(l)==FALSE, ]
  meta_plate <- l[duplicated(l[, c("plate", "plate_id")])==FALSE, 
                  c("plate", "plate_id")]
  meta_group <- l[duplicated(l[, c("group", "group_id")])==FALSE, 
                  c("group", "group_id", "compound", "dose", 
                    "plate_id", "plate")]
  meta_plate_group <- l[duplicated(l[, c("group", "group_id", 
                                         "plate", "plate_id",
                                         "plate_group", 
                                         "plate_group_id")])==FALSE, 
                        c("well_id", "group", "group_id", 
                          "plate", "plate_id",
                          "compound", "dose", 
                          "plate_group", "plate_group_id")]
  
  # par: alpha_p
  alpha_p <- data.frame(summary(f, par = "alpha_p")$summary)
  alpha_p$plate_id <- seq_len(nrow(alpha_p))
  alpha_p <- merge(x = alpha_p, y = meta_plate, 
                   by = "plate_id", all.x = TRUE)
  
  # par: mu_group
  mu_group <- data.frame(summary(f, par = "mu_group")$summary)
  mu_group$group_id <- seq_len(nrow(mu_group))
  mu_group <- merge(x = mu_group, y = meta_group, by = "group_id", all.x = TRUE)
  
  # par: mu_plate_group
  mu_plate_group <- data.frame(summary(f, par = "mu_plate_group")$summary)
  mu_plate_group$plate_group_id <- seq_len(nrow(mu_plate_group))
  mu_plate_group <- merge(x = mu_plate_group, y = meta_plate_group, 
                          by = "plate_group_id", all.x = TRUE)
  
  # par: mu
  mu_well <- data.frame(summary(f, par = "mu")$summary)
  mu_well$well_id <- seq_len(nrow(mu_well))
  mu_well <- merge(x = mu_well, y = meta_well, by = "well_id", all.x = TRUE)
  
  # par: kappa
  kappa_well <- data.frame(summary(f, par = "kappa")$summary)
  kappa_well$well_id <- seq_len(nrow(kappa_well))
  kappa_well <- merge(x = kappa_well, y = meta_well, 
                      by = "well_id", all.x = TRUE)
  
  # par: sigma_bio
  sigma_bio <- data.frame(summary(f, par = "sigma_bio")$summary)
  sigma_tech <- data.frame(summary(f, par = "sigma_tech")$summary)
  
  # kappa mu and sigma
  kappa_mu <- data.frame(summary(f, par = "kappa_mu")$summary)
  kappa_sigma <- data.frame(summary(f, par = "kappa_sigma")$summary)
  
  # par: y_hat_sample
  yhat <- data.frame(summary(f, par = "y_hat_sample")$summary)
  yhat$well_id <- seq_len(nrow(yhat))
  yhat <- merge(x = yhat, y = meta_well, by = "well_id", all.x = TRUE)
  
  return(list(alpha_p = alpha_p,
              delta_t = mu_group, 
              delta_tp = mu_plate_group,
              well_mu = mu_well,
              well_kappa = kappa_well,
              kappa_mu = kappa_mu,
              kappa_sigma = kappa_sigma,
              sigma_bio = sigma_bio,
              sigma_tech = sigma_tech,
              yhat = yhat))
}

###### testing x #######

test_that("x parameter takes invalid", {
  expect_error(cellmig(), 
               regexp = "x is missing or NULL")
  expect_error(cellmig(x = NULL), 
               regexp = "x is missing or NULL")
  expect_error(cellmig(x = NA), 
               regexp = "x must be data.frame")
  expect_error(cellmig(x = 0), 
               regexp = "x must be data.frame")
  expect_error(cellmig(x = F), 
               regexp = "x must be data.frame")
  expect_error(cellmig(x = data.frame()), 
               regexp = "zero or one row in x")
})


test_that("x parameter takes column values", {
  data("d", package = "cellmig")
  # use only one well (one compound)
  expect_error(cellmig(x = d[d$well=="1",]), 
               regexp = "only one compound included")
  
  expect_error(cellmig(x = d[d$compound=="C1",]), 
               regexp = "only one compound included")
  
  data("d", package = "cellmig")
  d$offset[d$offset==1] <- 2
  expect_error(cellmig(x = d),
               regexp = "column offset can contain only 0 or 1")
  
  data("d", package = "cellmig")
  d$offset[d$offset==1] <- NA
  expect_error(cellmig(x = d),
               regexp = "column offset can contain only 0 or 1")
  
  data("d", package = "cellmig")
  d$offset[d$offset==1] <- Inf
  expect_error(cellmig(x = d),
               regexp = "column offset can contain only 0 or 1")
  
  data("d_mini", package = "cellmig")
  d_mini$offset <- 0
  expect_no_error(cellmig(x = d_mini,
                       control = list(mcmc_warmup = 200,
                                      mcmc_steps = 900,
                                      mcmc_chains = 2,
                                      mcmc_cores = 2)))
})


test_that("x parameter takes column values", {
  data("d", package = "cellmig")
  d <- d[d$compound %in% c("C1", "C2"),]
  d <- d[d$plate == "1", ]
  o <- suppressWarnings(cellmig(x = d, control = list(mcmc_chains = 1,
                                                      mcmc_warmup = 200,
                                                      mcmc_steps = 500)))
  expect_length(object = o, n = 4)
  
  data("d", package = "cellmig")
  d <- d[d$compound %in% c("C1", "C2"),]
  o <- suppressWarnings(cellmig(x = d, control = list(mcmc_chains = 1,
                                                      mcmc_warmup = 200,
                                                      mcmc_steps = 500)))
  expect_length(object = o, n = 4)
})

---
title: "cellmig: quantifying cell migration with hierarchical Bayesian models"
author: "Simo Kitanovski (simo.kitanovski@uni-due.de)"
output:
  BiocStyle::html_document
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{User Manual: cellmig}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r setup, include = FALSE, warning = FALSE}
knitr::opts_chunk$set(comment = FALSE, 
                      warning = FALSE, 
                      message = FALSE)
```

```{r}
library(cellmig)
library(ggplot2)
library(ggforce)
ggplot2::theme_set(new = theme_bw(base_size = 10))
```

# Background

High-throughput tracking of cells with time-lapse microscopy followed by
the acquisition of images at ﬁxed time intervals facilitates the
analysis of cell migration across many wells treated under different
biological conditions. These workflows generate considerable technical
noise and biological variability, and therefore technical and biological
replicates are necessary, leading to large, hierarchically structured
datasets, i.e., cells are nested within technical replicates that are
nested within biological replicates.

Current statistical analyses of such data usually ignore the
hierarchical structure of the data and fail to explicitly quantify
uncertainty arising from technical or biological variability. To address
this gap, we present `r Biocpkg("cellmig")`, an R package implementing
Bayesian hierarchical models for migration analysis.
`r Biocpkg("cellmig")` quantifies condition-specific velocity changes
(e.g., drug effects) while modeling nested data structures and technical
artifacts, providing uncertainty-aware estimates through credible
intervals.

There are currently no Bioconductor packages providing specialized
statistical methods for analyzing hierarchical high-throughput cell
migration data. `r Biocpkg("cellmig")` addresses this gap and will
represent a valuable addition to the ecosystem.


# Installation
To install this package, start R (version "4.5") and enter:

```{r, eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("cellmig")
```


# Data

This is how a typical cell migration data looks like $\rightarrow$ a
table.

Each rows is a cell with the following features:

-   well = unique well ID (w1, w2, w3, etc.).
-   plate = unique plate ID (p1, p2, p3, etc.). Each plate is a
    biological replicate. A plate contains multiple wells, some of which
    are treated with the same compound and dose (technical replicates)
-   compound = compound name (c1, c2, c3, etc.)
-   dose = compound concentration (0, 1, 5, 10, low, mid, high, etc.)
-   v = Observed cell migration velocity (numeric)
-   offset = binary (0 or 1). Indicates whether a treatment should be
    used for batch correction across plates. By default offset = 0 (no
    correction). Set to 1 for specific treatment groups (compound x
    dose) used as offsets. Ensure that this treatment group appears on
    each plate.

```{r}
data("d", package = "cellmig")
str(d)
```

In this vignette we will use simulated data from:

-   **plates** ($p$): 1, 2, ... , 3
-   **wells** ($w$): 1, 2, ... , 378
-   **cells** per well with their migration velocity `v`
-   wells are treated with **compounds** 1, 2, ..., 6 at **dose** 1, 2,
    ..., 7.
-   combination of a compound and dose is a **treatment group** ($t$)
    $\rightarrow$ 1, 2, ..., 42.

Let's visualize the data. Each dot represents a cell with its velocity
on the y-axis. Each facet corresponds to a compound (e.g., specific drug
that may affect cellular velocity). The x-axis represents the dose.
There are three plates, indicated by color. Four technical replicates
(wells), analyzed on the same plate, are stacked next to each other and
have the same color.

```{r, fig.width=7, fig.height=6}
ggplot(data = d)+
  facet_wrap(facets = ~paste0("compound=", compound), 
             scales = "free_y", ncol = 2)+
  geom_sina(aes(x = as.factor(dose), col = plate, y = v, group = well), 
            size = 0.5)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  ylab(label = "migration velocity")+
  xlab(label = '')+
  scale_color_grey()+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  guides(shape = guide_legend(override.aes = list(size = 3)))+
  scale_y_log10()+
  annotation_logticks(base = 10, sides = "l")
```

## **Mean migration velocity** per well

Alternatively, we can visualize the well-specific mean velocities to
highlight plate-specific batch effects.

```{r, fig.width=7, fig.height=6}
dm <- aggregate(v~well+plate+compound+dose, data = d, FUN = mean)
ggplot(data = dm)+
  facet_wrap(facets = ~paste0("compound=", compound), 
             scales = "free_y", ncol = 2)+
  geom_sina(aes(x = as.factor(dose), col = plate, y = v, group = well), 
            size = 1.5, alpha = 0.7)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  ylab(label = "migration velocity")+
  xlab(label = '')+
  scale_color_grey()+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  guides(shape = guide_legend(override.aes = list(size = 3)))+
  scale_y_log10()+
  annotation_logticks(base = 10, sides = "l")
```

# `cellmig` analysis

We will use this data to infer the **overall treatment effects**
(parameter $\delta_t$), relative to a control treatment (the offset) to
correct for plate-specific batch effects. At the same time,
`r Biocpkg("cellmig")` will quantify many different features of the data
using its model parameters (e.g., variability between technical or
biological replicates; or plate-specific treatment effects
($\gamma_{pt}$)).

## Model fitting

We fit the Stan model employed by `r Biocpkg("cellmig")` with the
control parameters defined in the list `control`. There are many other
input parameters in `control`, check the `cellmig` function documentation.

```{r, fig.width=7, fig.height=3.5}
o <- cellmig(x = d,
             control = list(mcmc_warmup = 300, # nr. of MCMC warmup step?
                            mcmc_steps = 1000, # nr. of MCMC iteration steps?
                            mcmc_chains = 2,   # nr. of MCMC chains
                            mcmc_cores = 2))   # nr. of MCMC cores
```

## What are the **overall treatment effects** ($\delta_t$) on velocity?

To extract the means, medians, and 95% Highest Density Intervals (HDIs,
quantifying parameter value uncertainty) of $\delta_t$, we have to
access the data.frame `delta_t` in the output object `posteriors`:

```{r}
str(o$posteriors$delta_t)
```

It is better to visualize the mean $\delta_t$s and their 95% HDIs

-   Dot: Posterior mean of $\delta_t$
-   Error bar: 95% highest density interval (HDI) of $\delta$
-   $\exp(\delta)$: Fold change in cell velocity relative to control

As compound t=1 was selected as control (by setting offset=1), the
treatment effects of this compounds are not shown.

```{r, fig.width=6, fig.height=3.3}
ggplot(data = o$posteriors$delta_t)+
  geom_line(aes(x = dose, y = mean, col = compound, group = compound))+
  geom_point(aes(x = dose, y = mean, col = compound))+
  geom_errorbar(aes(x = dose, y = mean, ymin = X2.5., ymax = X97.5., 
                    col = compound), width = 0.1)+
  ylab(label = expression("Overall treatment effect ("*delta*")"))+
  theme(legend.position = "top")
```

## Compare the dose-response `profiles` for different compounds

For "rectangular datasets", i.e. datasets with multiple compounds and
overlapping doses, we can study the treatment dose-response profiles by
hierarchical clustering based on the complete posteriors of $\delta_t$,
account for uncertainty in this parameter.

Panel A: dendrogram constructed by hierarchical clustering with average
linkage, based on euclidean distances between vectors of $\delta_t$
(shown in panel B) of each compound (leaf) across doses. Branch support
values show branch robustness (label = 1000 implies this branch was
encountered in each of the 1000 dendrograms constructed from the
posterior of $\delta_t$). Plate-specific treatment dose-responses based
on parameters $\gamma_pt$.

-   Dot in panel B/C: Posterior mean of $\delta_t$ and $\gamma_pt$
-   Error bar: 95% highest density interval (HDI)

```{r, fig.width=9.5, fig.height=5}
get_dose_response_profile(x = o)+
  patchwork::plot_layout(widths = c(.7, 1, 4))
```

## Compare the effects between treatment group

Pairwise dot-plot comparison $\rightarrow$ x minus y axis

(Left panel) Differences in overall treatment effects. Log fold change
(LFC; described by parameter $\rho_{ij}$) between overall treatments
effects ($\delta_t$) of row ($i$) vs. column ($j$) treatment groups.
Tile colors and labels represent $\rho_{ij}$. (Right panel) Probability
of differential treatment effect described by parameter $\pi_{ij}$. Tile
colors and labels represent $\pi_{ij}$.

-   x/y-axis treatment groups (combinations of compounds and doses)
-   $\rho$: Difference between treatment groups at y-x axis.
-   $\pi$: probability of observing either a completely positive or
    negative $\rho$

```{r, fig.width=14, fig.height=6}
u <- get_pairs(x = o, exponentiate = FALSE)
u$plot
```

## Violin plot based comparison

-   from_groups: vector of treatment groups to consider (combinations of
    compounds and doses)
-   to_group: target treatment group
-   violins show the posterior distributions of the differences ($\rho$:
    each element from `from_groups` vs. `to_group`).
-   label: probability, $\pi$, of observing completely positive or
    negative $\rho$

```{r, fig.width=7, fig.height=2.5}
get_groups(x = o)
u <- get_violins(x = o, 
                 from_groups = get_groups(x = o)$group,
                 to_group = "C2|D1",
                 exponentiate = FALSE)
u$plot
```

## Posterior predictive checks (PPCs)

To assess model validity, we performed posterior predictive checks,
which showed that the simulated data (pink violin) were consistent with
the observed data (black violins). Each dot is a cells.

```{r, fig.width=5, fig.height=6}
g <- get_ppc_violins(x = o, wrap = TRUE, ncol = 3)
g+scale_y_log10()
```

Using posterior predictive checks we compared the mean simulated
velocity per well (y-axis) with the observed mean per well (x-axis).
Each dot is a well.

```{r, fig.width=4, fig.height=4}
g <- get_ppc_means(x = o)
g
```

## Inspecting other model parameters

```{r, fig.height=2, fig.width=6}
g_alpha_p <- ggplot(data = o$posteriors$alpha_p)+
  geom_errorbarh(aes(y = plate_id, x = mean, xmin = X2.5., xmax = X97.5.),
                 height = 0.1)+
  geom_point(aes(y = plate_id, x = mean))

g_sigma <- ggplot()+
  geom_errorbarh(data = o$posteriors$sigma_bio,
                 aes(y = "sigma_bio",
                     x = mean, xmin = X2.5., xmax = X97.5.),
                 height = 0.1)+
  geom_errorbarh(data = o$posteriors$sigma_tech,
                 aes(y = "sigma_tech",
                     x = mean, xmin = X2.5., xmax = X97.5.),
                 height = 0.1)+
  geom_point(data = o$posteriors$sigma_bio,
             aes(y = "sigma_bio", x = mean))+
  geom_point(data = o$posteriors$sigma_tech,
             aes(y = "sigma_tech", x = mean))+
  ylab(label = '')

g_alpha_p|g_sigma
```

# Session Info

```{r}
sessionInfo()
```

---
title: "Simulating data with cellmig"
author: "Simo Kitanovski (simo.kitanovski@uni-due.de)"
output:
  BiocStyle::html_document
vignette: >
  %\VignetteEncoding{UTF-8}
  %\VignetteIndexEntry{User manual: data simulation}
  %\VignetteEngine{knitr::rmarkdown}
editor_options: 
  markdown: 
    wrap: 72
---

```{r setup, include = FALSE, warning = FALSE}
knitr::opts_chunk$set(comment = FALSE, 
                      message = FALSE)
```

```{r}
library(cellmig)
library(ggplot2)
library(ggforce)
library(patchwork)
library(rstan)

ggplot2::theme_set(new = theme_bw(base_size = 10))
```

# Background

High-throughput cell migration assays generate complex, hierarchically
structured datasets with inherent technical noise and biological
variability. To accurately quantify treatment effects on cell migration
velocity, it is essential to account for these sources of variation and
to design experiments that provide sufficient statistical power. The
`r Biocpkg("cellmig")` R package implements a Bayesian hierarchical model 
for analyzing such data. See the "User manual" vignette to understand the
key functionalities of `r Biocpkg("cellmig")`.

Furthermore, `r Biocpkg("cellmig")` includes functionality for simulating
synthetic datasets. These simulations are invaluable for experimental
planning, allowing researchers to assess how different experimental
designs (e.g., varying numbers of biological replicates, technical
replicates, or cells per well) affect the precision of treatment effect
estimates.

This vignette demonstrates how to use `r Biocpkg("cellmig")` to simulate 
cell migration data under a partially generative model. These functionalities
are documented in this vignette. In short, we simulate data based on 
realistic parameter values derived from experimental data and
then analyze the simulated data using `r Biocpkg("cellmig")` to recover the
parameters. This process validates the model and illustrates how
simulation can guide experimental design and power analysis.


# Installation
To install this package, start R (version "4.5") and enter:

```{r, eval=FALSE}
if (!require("BiocManager", quietly = TRUE))
    install.packages("BiocManager")

BiocManager::install("cellmig")
```

# Data simulation from **partially generative** model

We simulate data for an experiment with the following design:

-   Six biological replicates (plates), indexed by $p = 1, 2, \dots, 6$.

-   Three technical replicates (wells) per treatment per plate, indexed
    by $t = 1, 2, 3$.

-   30 cells per well.

-   Seven treatment groups ($t = \{1, \dots, 7\}$) with fixed effects
    $\delta_t = [-0.4, -0.2, -0.1, 0, 0.1, 0.2, 0.4]$.

-   Biological replicate variability: $\sigma_{\text{bio}} = 0.1$.

-   Technical replicate variability: $\sigma_{\text{tech}} = 0.05$.

## Model specification

### Setting $\alpha_p$

Plate-specific batch effects are modeled by drawing the plate offset
$\alpha_p$ from a normal distribution:

$$
\alpha_p \sim \text{Normal}(1.7, 0.1)
$$

Here, $\alpha_p$ represents the log-transformed mean velocity of the
control treatment on plate $p$. The mean (1.7) and standard deviation
(0.1) are chosen based on experimental data. If batch effects are
minimal, the standard deviation can be set to a smaller value (e.g.,
0.01).

### Setting $\kappa_w$

The velocities in each well are gamma distributed, and the gamma
distribution is defined by a rate and shape parameter. The shape
parameter for well $w$, $\kappa_w$, controls the form or skewness of the
distribution and, to some extent, the variability in the data.

We simulate the values of $\log(\kappa_w)$ by drawing randomly from a
normal distribution with mean ($\mu_{\kappa}$) and standard deviation
(${\sigma}_{\kappa}$). The values of $\mu_{\kappa}$ and
${\sigma}_{\kappa}$ can either be fixed to point values, or be drawn
from two normal distributions whose means and standard deviations are
described by the inputs `prior_kappa_mu_M`, `prior_kappa_sigma_M`,
`prior_kappa_mu_SD`, and `prior_kappa_sigma_SD`. To fix the values of
$\mu_{\kappa}$ and ${\sigma}_{\kappa}$ to point estimates set the scales
to small values (e.g., set in `control` `prior_kappa_mu_SD`=0.01 and
`prior_kappa_sigma_SD`).

### Defining the offset

To correct for batch effects across plates, we designate one treatment
as the reference (offset). Here, we set the offset to treatment 4 (i.e.,
the fourth treatment group, which has $\delta_t = 0$).

## Generates dataframe with simulated velocities

We use the `gen_partial` function from `r Biocpkg("cellmig")` to simulate the 
data. The function requires a control list specifying the experimental design
and model parameters.

```{r}
# set seed for reproducible results
set.seed(seed = 1253)
# simulate data
y_p <- gen_partial(control = list(N_biorep = 8, 
                                  N_techrep = 5, 
                                  N_cell = 50, 
                                  delta = c(-0.4, -0.2, -0.1, 0, 0.1, 0.2, 0.4),
                                  sigma_bio = 0.1, 
                                  sigma_tech = 0.05, 
                                  offset = 4,
                                  prior_alpha_p_M = 1.7,
                                  prior_alpha_p_SD = 0.1,
                                  prior_kappa_mu_M = 1.7,
                                  prior_kappa_mu_SD = 0.1,
                                  prior_kappa_sigma_M = 0,
                                  prior_kappa_sigma_SD = 0.1))
```

```{r}
# see data structure
str(y_p$y)
```

The output is a dataframe with columns:

-   `v`: Cell velocity

-   `well`: Well identifier

-   `plate`: Plate identifier

-   `group`: Treatment group

-   `compound`: Compound identifier (same as group in this simulation)

-   `dose`: Dose level (not used in this simulation, set to "X")

**Visualizing the simulated data**

We can visualize the simulated cell velocities using scatter plots, with
points jittered by well and colored by plate.

```{r, fig.width=7, fig.height=3}
ggplot(data = y_p$y)+
  geom_sina(aes(x = paste0("t=", group), col = paste0("p=", plate), 
                y = v, group = well), size = 0.5)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  xlab(label = "dose")+
  ylab(label = "migration speed")+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  scale_color_grey(name = "plate")+
  scale_y_log10()
```

Alternatively, we can visualize the well-specific mean velocities to
highlight plate-specific batch effects:

```{r, fig.width=7, fig.height=3}
ggplot(data = aggregate(v~well+group+plate, data = y_p$y, FUN = mean))+
  geom_sina(aes(x = paste0("t=", group), col = paste0("p=", plate), 
                y = v, group = well), size = 1)+
  theme_bw()+
  theme(legend.position = "top",
        strip.text.x = element_text(margin = margin(0.03,0,0.03,0, "cm")))+
  xlab(label = "dose")+
  ylab(label = "migration speed")+
  guides(color = guide_legend(override.aes = list(size = 3)))+
  scale_y_log10()+
  scale_color_grey(name = "plate")
```

### Analyzing simulated data with `r Biocpkg("cellmig")`

The simulated data must be formatted to match the input requirements of
`r Biocpkg("cellmig")`. Specifically, we need to convert some columns to 
character and add an `offset` column indicating the reference treatment.

```{r, fig.width=7, fig.height=3.5}
sim_data <- y_p$y

# format simulated data to be used as input in cellmig
sim_data$well <- as.character(sim_data$well)
sim_data$compound <- as.character(sim_data$compound)
sim_data$plate <- as.character(sim_data$plate)
sim_data$offset <- 0
sim_data$offset[sim_data$group=="4"] <- 1
```

We now analyze the simulated data using `r Biocpkg("cellmig")` with the 
following control parameters:

```{r, fig.width=7, fig.height=3.5}
osd <- cellmig(x = sim_data,
               control = list(mcmc_warmup = 250,
                              mcmc_steps = 800,
                              mcmc_chains = 2,
                              mcmc_cores = 2,
                              mcmc_algorithm = "NUTS",
                              adapt_delta = 0.8,
                              max_treedepth = 10))
```

### Comparing simulated vs. inferred parameters

We compare the inferred parameters (posterior means and 95% HDIs) with
the true values used in the simulation.

#### Plate-specific offsets ($\alpha_p$)

```{r, fig.width=4, fig.height=2}
ggplot(data = osd$posteriors$alpha_p)+
  geom_point(aes(y = plate_id, x = exp(mean)))+
  geom_errorbarh(aes(y = plate_id, x = exp(mean), xmin = exp(X2.5.), 
                     xmax = exp(X97.5.)), height = 0.1)+
  geom_point(data = data.frame(v = exp(y_p$par$alpha_p),
                               plate = osd$posteriors$alpha_p$plate_id),
             aes(y = plate, x = v), col = "red")+
  scale_y_continuous(breaks = osd$posteriors$alpha_p$plate_id)+
  xlab(label = expression(alpha[p]*"'"))
```

#### Effect sizes ($\delta_t$)

```{r, fig.width=4, fig.height=2.5}
ggplot(data = osd$posteriors$delta_t)+
  geom_point(aes(y = group_id, x = mean))+
  geom_errorbarh(aes(y = group_id, x = mean, xmin = X2.5., xmax = X97.5.), 
                 height = 0.1)+
  geom_point(data = data.frame(delta = c(-0.4, -0.2, -0.1, 0.1, 0.2, 0.4), 
                               group_id = 1:6),
             aes(y = group_id, x = delta), col = "red")+
  xlab(label = expression(delta[t]))+
  ylab(label = "treatment")+
  scale_y_continuous(breaks = 1:8)
```

#### Biological ($\sigma_{bio}$) and technical ($\sigma_{tech}$) variability 

```{r, fig.width=4, fig.height=2}
plot(osd$fit, par = c("sigma_bio", "sigma_tech"))
```



# Data simulation from **fully generative** model 
We can also use `r Biocpkg("cellmig")` to generate cell velocities from 
the prior distributions of the parameters $\rightarrow$ data from prior 
predictive distribution.

In this scenario, the parameters, such as $\delta_t$, will not be fixed to
point values but rather generated from their priors.

## Model specification

```{r}
control <- list(N_biorep = 4, 
                N_techrep = 3, 
                N_cell = 30, 
                N_group = 8,
                prior_alpha_p_M = 1.7,
                prior_alpha_p_SD = 1.0,
                prior_kappa_mu_M = 1.7,
                prior_kappa_mu_SD = 1.0,
                prior_kappa_sigma_M = 0,
                prior_kappa_sigma_SD = 1.0,
                prior_sigma_bio_M = 0.0,
                prior_sigma_bio_SD = 1.0,
                prior_sigma_tech_M = 0.0,
                prior_sigma_tech_SD = 1.0,
                prior_mu_group_M = 0.0,
                prior_mu_group_SD = 1.0)
```



```{r}
# generate data
y_f <- gen_full(control = control)
```

The generate data has a similar structure as before:

```{r}
str(y_f)
```



## Compare density of velocities from partially vs. fully generative models

```{r, fig.width=4, fig.height=4}
w <- data.frame(v_f = y_f$y$v[1:2000], v_p = y_p$y$v[1:2000])
 ggplot(data = w)+
    geom_point(aes(x = v_f, y = v_p), size = 0.5)+
    geom_density_2d(aes(x = v_f, y = v_p), col = "orange")+
    scale_x_log10(name = "Velocity from fully generative model", 
                  limits = c(0.01, 10^4))+
    scale_y_log10(name = "Velocity from partially generative model", 
                  limits = c(0.01, 10^4))+
    annotation_logticks(base = 10, sides = "lb")+
    theme_bw(base_size = 10)
```

# Power analysis
How many biological replicates do we need to capture certain effect size 
($\delta$)? Similar question can be asked in order to optimize the number 
of technical replicates or the number of cells per well.

The code below takes considerable time to be executed. This is because we 
generate N_biorep times N_sim datasets and fit a model. To avoid this during
R-package checks, the following chunks will not be evaluated. Do this 
yourselves.

**Important note**: using multicore execution you can considerably speed-up 
this analysis!

```{r, eval = FALSE, message=FALSE, warning=FALSE}
# Constants
N_bioreps <- c(3, 6, 9)
N_sim <- 10
true_deltas <- c(-0.3, -0.15, 0, 0.2, 0.4)
offset <- 3

# power analysis
deltas <- vector(mode = "list", length = length(N_bioreps)*N_sim)
i <- 1
for(N_biorep in N_bioreps) {
  for(b in 1:N_sim) {
    # simulate data
    y_p <- gen_partial(control = list(N_biorep = N_biorep, 
                                      N_techrep = 3, 
                                      N_cell = 40, 
                                      delta = true_deltas,
                                      sigma_bio = 0.1, 
                                      sigma_tech = 0.05, 
                                      offset = offset,
                                      prior_alpha_p_M = 1.7,
                                      prior_alpha_p_SD = 0.1,
                                      prior_kappa_mu_M = 1.7,
                                      prior_kappa_mu_SD = 0.1,
                                      prior_kappa_sigma_M = 0,
                                      prior_kappa_sigma_SD = 0.1))
    
    # format simulated data to be used as input in cellmig
    sim_data <- y_p$y
    sim_data$well <- as.character(sim_data$well)
    sim_data$compound <- as.character(sim_data$compound)
    sim_data$plate <- as.character(sim_data$plate)
    sim_data$offset <- 0
    sim_data$offset[sim_data$group==offset] <- 1
    
    # run cellmig
    o <- cellmig(x = sim_data,
                   control = list(mcmc_warmup = 300,
                                  mcmc_steps = 1000,
                                  mcmc_chains = 1,
                                  mcmc_cores = 1,
                                  mcmc_algorithm = "NUTS",
                                  adapt_delta = 0.8,
                                  max_treedepth = 10))
    
    delta <- o$posteriors$delta_t
    delta$b <- b
    delta$N_biorep <- N_biorep
    delta$true_deltas <- true_deltas[-offset]
    delta$TP <- (delta$X2.5. <= delta$true_deltas & 
                   delta$X97.5. >= delta$true_deltas) &
      !(delta$X2.5. <= 0 & delta$X97.5. >= 0)
    deltas[[i]] <- delta
    i <- i + 1
  }
}

rm(i, delta, o, sim_data, y_p, b, N_biorep, offset, true_deltas)

deltas <- do.call(rbind, deltas)
```

What do you see? For Large effect ($|\delta_t|>>0$) we need few biological 
replicates to capture the true effect and not the null effect ($\delta_t=0$),
i.e., in nearly all simulations we have a true positive. For smaller effects,
this is not the case, and we probably need many more biological replicates
to each a high true positive rate.

```{r, eval = FALSE, fig.height=3, fig.width = 6}
ggplot(data = aggregate(TP~N_biorep+true_deltas, data = deltas, FUN = sum))+
  geom_point(aes(x = N_biorep, y = TP, col = abs(true_deltas), 
                 group = as.factor(true_deltas)), size = 2, alpha = 0.5)+
  geom_line(aes(x = N_biorep, y = TP, col = abs(true_deltas), 
                group = as.factor(true_deltas)), alpha = 0.5)+
  ylab(label = "Number of true positives (TPs)")+
  scale_color_distiller(name = expression("|"*delta[t]*"|"),palette="Spectral")+
  theme_bw(base_size = 10)
```


# Session Info

```{r}
sessionInfo()
```

\name{get_dose_response_profile}
\alias{get_dose_response_profile}
\title{Visualization of dose-response profiles}
\usage{
get_dose_response_profile(x,
                          hc_link = "average",
                          hc_dist = "euclidean",
                          groups,
                          B = 1000)
}
\arguments{
  \item{x}{An object generated by \code{cellmig}}
  \item{hc_link}{hierarchical clustering linkage function}
  \item{hc_dist}{hierarchical clustering distance metric}
  \item{groups}{use only selected treatment groups}
  \item{B}{number of samples to draw from posterior}
}
\value{
A patchwork plot containing:
\itemize{
  \item \strong{A}: Hierarchical dendrogram comparing compound effect curves.
  \item \strong{B}: Mean effects of compounds and doses on cell migration 
  with 95\% credible intervals.
  \item \strong{C}: Mean effects of compounds and doses on cell migration 
  in each replicate with 95\% credible intervals.
}
}
\description{
The function takes as its main input (\code{x}) the output of the 
\code{cellmig} function. Users can select a subset of treatment groups
(compounds + dose combinations) using \code{groups}. To construct the
hierarchical dendrogram, one has to select a distance metric, 
\code{hc_dist}, (e.g. \code{hc_dist} = "euclidean"); and a linkage 
function, \code{hc_link}, (e.g. \code{hc_link} = "average").
}
\details{
The function \code{get_dose_response_profile} visualizes compound effect profiles, 
identifying compounds with similar effects on cell migration across 
different doses. These compounds are clustered together in the 
hierarchical dendrogram.
}
\examples{
data(d, package = "cellmig")
o <- cellmig(x = d,
            control = list(mcmc_warmup = 200,
                           mcmc_steps = 500,
                           mcmc_chains = 2,
                           mcmc_cores = 2,
                           mcmc_algorithm = "NUTS",
                           adapt_delta = 0.8,
                           max_treedepth = 10))

p <- get_dose_response_profile(x = o,
                               hc_link = "average",
                               hc_dist = "euclidean",
                               B = 100)
p
}

\name{get_groups}
\alias{get_groups}
\title{Extract group labels}
\description{
  This function extracts treatment group metadata from an object generated by
  \code{cellmig}.
}
\usage{
  get_groups(x)
}
\arguments{
  \item{x}{An object generated by \code{cellmig}, containing treatment group
  names and IDs, compound names and dose.}
}
\details{
  The function retrieves group metadata such as group identifiers, names, 
  compound information, and dose levels.
}
\value{
  A data frame containing the following columns:
  \item{group_id}{Unique identifier for each treatment group.}
  \item{group}{Name of the treatment group.}
  \item{compound}{Compound associated with the treatment group.}
  \item{dose}{Dose level of the compound.}
}
\examples{
data(d_mini, package = "cellmig")
o <- cellmig(x = d_mini,
             control = list(mcmc_warmup = 200,
                            mcmc_steps = 500,
                            mcmc_chains = 2,
                            mcmc_cores = 2,
                            mcmc_algorithm = "NUTS",
                            adapt_delta = 0.8,
                            max_treedepth = 10))
u <- get_groups(x = o)
}
\seealso{
  \code{cellmig}, \code{get_pairs}, \code{get_groups}, \code{get_violins}
}

\name{gen_partial}
\alias{gen_partial}
\title{Simulating data from partially generative Bayesian model}
\description{
  Simulates cell velocities based on a hierarchical model using a Stan model,
  where some of the model parameter values are defined (fixed) by the user,
  while the rest are drawn from the prior distributions.
}
\usage{
  gen_partial(control = list(N_biorep = 3, 
                             N_techrep = 3, 
                             N_cell = 50, 
                             delta,
                             sigma_bio = 0.1, 
                             sigma_tech = 0.05, 
                             offset = 1,
                             prior_alpha_p_M = 1.7,
                             prior_alpha_p_SD = 0.5,
                             prior_kappa_mu_M = 1.7,
                             prior_kappa_mu_SD = 0.5,
                             prior_kappa_sigma_M = 0,
                             prior_kappa_sigma_SD = 0.3))
}
\arguments{
  \item{control}{A list configuring the MCMC sampling algorithm, and parameter
priors with the following default values:
\itemize{
  \item{N_biorep: Number of plates (biological replicates) in the simulation.}
  \item{N_techrep: Number of technical replicates: wells on a plate treated 
  with the same treatment.}
  \item{N_cell: Number of cells per well.}
  \item{delta: Effects (on log-scale) on cell velocity for each treatment.}
  \item{sigma_bio: Variability between biological replicates.}
  \item{sigma_tech: Variability between technical replicates.}
  \item{offset: Index of the control treatment (which treatment should be used
  for batch correction between plates).}
  \item{prior_alpha_p_M, prior_alpha_p_SD: Mean (M) and standard deviation 
  (SD) of a normal distribution describing plate-specific batch effects 
  (on log-scale).}
  \item{prior_kappa_mu_M, prior_kappa_mu_SD: Mean (M) and standard deviation 
  (SD) of a normal distribution describing the mean of the population of 
  well-specific shape (kappa) parameters.}
  \item{prior_kappa_sigma_M, prior_kappa_sigma_SD: Mean (M) and standard 
  deviation (SD) of a normal distribution describing the standard deviation 
  of the population of well-specific shape (kappa) parameters.}
}
}
}
\details{
  This function constructs a hierarchical dataset by simulating values 
  using a partially generative Stan model. It generates metadata for each 
  well and simulates data using the `sampling` function from the `rstan` 
  package.
}
\value{
  A data frame containing:
  \itemize{
    \item{iteration = Simulation iteration index.}
    \item{well_id = Unique well identifier.}
    \item{y = Simulated response value.}
    \item{group_id = Experimental group identifier.}
    \item{plate_id = Plate identifier.}
  }
}
\examples{
g <- gen_partial(control = list(N_biorep = 3, 
                                N_techrep = 3, 
                                N_cell = 50, 
                                delta=c(0, -0.4, -0.2, -0.1, 0, 0.1, 0.2, 0.4),
                                sigma_bio = 0.2, 
                                sigma_tech = 0.05, 
                                offset = 1,
                                prior_alpha_p_M = 1.7,
                                prior_alpha_p_SD = 0.5,
                                prior_kappa_mu_M = 1.7,
                                prior_kappa_mu_SD = 0.5,
                                prior_kappa_sigma_M = 0,
                                prior_kappa_sigma_SD = 0.3))
str(g)
}

\name{gen_full}
\alias{gen_full}
\title{Simulating data from fully generative Bayesian model}
\description{
  Simulates cell velocities based on a hierarchical model using a Stan model,
  where some of the model parameters are drawn from their prior distributions.
}
\usage{
  gen_full(control = list(N_biorep = 3, 
                          N_techrep = 3,
                          N_cell = 50,
                          N_group = 5,
                          prior_alpha_p_M = 1.7,
                          prior_alpha_p_SD = 1,
                          prior_kappa_mu_M = 1.7,
                          prior_kappa_mu_SD = 1,
                          prior_kappa_sigma_M = 0,
                          prior_kappa_sigma_SD = 1,
                          prior_sigma_bio_M = 0,
                          prior_sigma_bio_SD = 1,
                          prior_sigma_tech_M = 0,
                          prior_sigma_tech_SD = 1,
                          prior_mu_group_M = 0,
                          prior_mu_group_SD = 1))
}
\arguments{
  \item{control}{A list configuring the MCMC sampling algorithm, and parameter
priors with the following default values:
\itemize{
  \item{N_biorep: Number of plates (biological replicates) in the simulation.}
  \item{N_techrep: Number of technical replicates: wells on a plate treated 
  with the same treatment.}
  \item{N_cell: Number of cells per well.}
  \item{prior_alpha_p_M, prior_alpha_p_SD: Mean (M) and standard deviation 
  (SD) of a normal distribution describing plate-specific batch effects 
  (on log-scale).}
  \item{prior_kappa_mu_M, prior_kappa_mu_SD: Mean (M) and standard deviation 
  (SD) of a normal distribution describing the mean of the population of 
  well-specific shape (kappa) parameters.}
  \item{prior_kappa_sigma_M, prior_kappa_sigma_SD: Mean (M) and standard 
  deviation (SD) of a normal distribution describing the standard deviation 
  of the population of well-specific shape (kappa) parameters.}
  \item{prior_sigma_bio_M,  prior_sigma_bio_SD: Mean (M) and standard 
  deviation (SD) of a normal distribution describing the variability of 
  overall treatment effects between biological replicates.}
  \item{prior_sigma_tech_M,  prior_sigma_tech_SD: Mean (M) and standard 
  deviation (SD) of a normal distribution describing the variability of
  treatment effects between technical replicates.}
  \item{prior_mu_group_M, prior_mu_group_SD: Mean (M) and standard 
  deviation (SD) of a normal distribution describing the population of overall 
  treatment effects.
}
}
}
}
\details{
  This function constructs a hierarchical dataset by simulating values 
  using a partially generative Stan model. It generates metadata for each 
  well and simulates data using the `sampling` function from the `rstan` 
  package.
}
\value{
  A data.frame containing cells as rows with:
  \itemize{
    \item{v = Simulated cell velocity.}
    \item{well_id = Unique well identifier.}
    \item{group_id = Experimental group identifier.}
    \item{plate_id = Biological replicate (plate) identifier.}
    \item{trep_id = Technical replicate identifier.}
  }
}
\examples{
f <- gen_full(control = list(N_biorep = 3, 
                             N_techrep = 3, 
                             N_cell = 50,
                             N_group = 5,
                             prior_alpha_p_M = 1.7,
                             prior_alpha_p_SD = 1.0,
                             prior_kappa_mu_M = 1.7,
                             prior_kappa_mu_SD = 1.0,
                             prior_kappa_sigma_M = 0,
                             prior_kappa_sigma_SD = 1.0,
                             prior_sigma_bio_M = 0.0,
                             prior_sigma_bio_SD = 1.0,
                             prior_sigma_tech_M = 0.0,
                             prior_sigma_tech_SD = 1.0,
                             prior_mu_group_M = 0.0,
                             prior_mu_group_SD = 1.0))
str(f)
}

\name{get_pairs}
\alias{get_pairs}
\title{Compare the cell migration effects of treatment groups (cellvel results)}
\usage{
get_pairs(x, groups, exponentiate)
}
\arguments{
\item{x}{an object generated by \code{cellvel}}
\item{groups}{vector of treatment groups to select, see groups by calling 
\code{get_groups}}
\item{exponentiate}{logical, should the effect be exponentiating turning
log-fold-changes into more interpretable fold-changes}
}
\value{
A data.frame with the following data in each row (comparison):
\itemize{
\item treatment group x
\item treatment group y
\item \eqn{\rho_M} = mean rho
\item \eqn{\rho_M}, \eqn{\rho_{L95}}, \eqn{\rho_{H95}} = mean, lower 
bound and upper bound of the difference between the migration effects of 
treatments x and y
\item \eqn{\pi} = probability of differential effects
}

Two heatmaps (ggplot2 objects) showing \eqn{\rho} (plot_rho) and \eqn{\pi} 
(plot_pi) between treatments (interpretation: rows vs. columns). 
}
\description{
The function \code{get_pairs} compares the overall treatment group effects 
with each other, i.e. computes differences in their posterior distributions.
}
\details{
The main input, \code{x}, of \code{get_pairs} is the output object from
the function \code{cellmig}.

For a pair of treatment groups, \code{get_pairs} compares extracts the 
posterior distributions of their cell migration effects, and then compute 
an absolute difference between the posteriors. The result is another 
posterior distribution (\eqn{\rho}), with L95 and H95 as the lower and 
upper bounds of the 95\% highest density interval of \eqn{\rho}.

\eqn{\pi} is estimated as: \eqn{\pi=2\cdot\max{(\int_{-\infty}^{0} \rho,
\int_{0}^{+\infty} \rho)}-1}.

For treatment groups, i and j, where \eqn{\rho<0} and the 95\% HDIs of
\eqn{\rho} lie mostly or completely below 0 (0 = null effect), we have
strong evidence of lower cell migration effect in treatment i compared to
j. On the other hand, for treatment groups, i and j, where \eqn{\rho} and 
the 95\% HDI of \eqn{\rho} lie mostly or completely above 0, we have strong 
evidence of higher cell migration effect in treatment i compared to j. 
Distributions with the 95\% HDIs more or less centered around 0 indicate 
that there is no evidence for a clear difference in the cell migration effects
between the treatment groups. Note that unclear evidence is not equivalent 
to no change, because for a treatment group with \eqn{\rho\approx 0} we 
may also have a wide 95\% HDI, including possibilities for positive or 
negative change.
}
\examples{
data(d_mini, package = "cellmig")
o <- cellmig(x = d_mini,
             control = list(mcmc_warmup = 200,
                            mcmc_steps = 500,
                            mcmc_chains = 2,
                            mcmc_cores = 2,
                            mcmc_algorithm = "NUTS",
                            adapt_delta = 0.8,
                            max_treedepth = 10))
u <- get_pairs(x = o, exponentiate = FALSE)
head(u)
}

\name{get_treatment_profile}
\alias{get_treatment_profile}
\title{Visualization of treatment effect profiles}
\usage{
get_treatment_profile(x,
                      hc_link = "average",
                      hc_dist = "euclidean",
                      groups,
                      B = 1000)
}
\arguments{
  \item{x}{An object generated by \code{cellmig}}
  \item{hc_link}{hierarchical clustering linkage function}
  \item{hc_dist}{hierarchical clustering distance metric}
  \item{groups}{use only selected treatment groups}
  \item{B}{number of samples to draw from posterior}
}
\value{
Hierarchical dendrogram comparing treatment group effects.
}
\description{
The function takes as its main input (\code{x}) the output of the 
\code{cellmig} function. Users can select a subset of treatment groups
(compounds + dose combinations) using \code{groups}. To construct the
hierarchical dendrogram, one has to select a distance metric, 
\code{hc_dist}, (e.g. \code{hc_dist} = "euclidean"); and a linkage 
function, \code{hc_link}, (e.g. \code{hc_link} = "average").
}
\details{
The function \code{get_treatment_profile} visualizes treatment group effect 
profiles: these treatment groups are clustered together in a hierarchical 
dendrogram.
}
\examples{
data(d, package = "cellmig")
o <- cellmig(x = d,
            control = list(mcmc_warmup = 200,
                           mcmc_steps = 500,
                           mcmc_chains = 2,
                           mcmc_cores = 2,
                           mcmc_algorithm = "NUTS",
                           adapt_delta = 0.8,
                           max_treedepth = 10))
                           
p <- get_treatment_profile(x = o,
                           hc_link = "average",
                           hc_dist = "euclidean",
                           B = 100)
p
}

\docType{package}
\name{cellmig-package}
\alias{cellmig-package}
\title{cellmig: quantifying cell migration with hierarchical Bayesian models}
\description{
\code{cellmig} implements Bayesian hierarchical models for migration analysis
and quantifies condition-specific (drug effects) effects on cell velocity 
while modeling nested data structures and technical artifacts, providing 
uncertainty-aware estimates through credible intervals.
}
\details{
This package contains functions for modeling, clustering, and visualization
of cell migration velocity from high-throughput migration assays.
}
\seealso{
Useful links:
\itemize{
  \item \url{https://github.com/snaketron/cellmig}
  \item Report bugs at \url{https://github.com/snaketron/cellmig/issues}
}
}

\author{
Authors and maintainers:
\itemize{
  \item Simo Kitanovski \email{simokitanovski@gmail.com} 
  (\href{https://orcid.org/0000-0003-2909-5376}{ORCID})
}
}

\examples{
# load package input data
data(d, package = "cellmig")
o <- cellmig(x = d,
             control = list(mcmc_warmup = 200,
                            mcmc_steps = 700,
                            mcmc_chains = 2,
                            mcmc_cores = 2,
                            mcmc_algorithm = "NUTS",
                            adapt_delta = 0.8,
                            max_treedepth = 10))
str(o)
}

\name{get_ppc_means}
\alias{get_ppc_means}
\title{Observed vs. predicted cell migration means in each well}

\usage{
get_ppc_means(x)
}

\arguments{
  \item{x}{The function takes as its main input (\code{x}) the output of the 
  \code{cellmig} function, containing posterior samples from a Bayesian model. 
  The posterior predictive means are stored in \code{x$s$yhat}, while the 
  observed migration values are extracted from \code{x$x$d}.}
}

\value{
A \code{ggplot2} object displaying:
\itemize{
  \item A scatter plot comparing observed vs. predicted migration means.
  \item 95\% highest density interval (HDI) error bars for the predicted means.
  \item A dashed reference line (\code{y = x}) for ideal agreement.
}
}

\description{
The \code{get_ppc_means} function visualizes the comparison between observed and 
posterior predicted migration means. The function aggregates observed migration 
values by well, merges them with posterior predictive means, and generates a 
scatter plot with error bars representing the 95\% HDI.
}

\details{
The function follows these steps:
\enumerate{
  \item Compute mean observed (scaled) cell migration in each well
  \item Compute mean predicted (scaled) cell migration in each well + 95\% credible 
  intervals
  \item Generate a \code{ggplot2} visualization with:
    \itemize{
      \item A dashed diagonal indicating perfect agreement.
      \item Scatter points representing observed vs. predicted means.
      \item Error bars (dark gray) showing the 95\% HDI for predictions.
    }
}
}

\examples{
# Load example dataset
data(d_mini, package = "cellmig")

# Run cellmig with MCMC sampling
o <- cellmig(x = d_mini,
             control = list(mcmc_warmup = 200,
                            mcmc_steps = 500,
                            mcmc_chains = 2,
                            mcmc_cores = 2,
                            mcmc_algorithm = "NUTS",
                            adapt_delta = 0.8,
                            max_treedepth = 10))

# Generate PPC means plot
p <- get_ppc_means(x = o)
print(p)
}

\name{get_violins}
\alias{get_violins}
\title{Generate violin plots for treatment group comparisons}
\description{
  This function generates violin plots for comparing posterior distributions 
  of treatment group effects on cell migration.
}
\usage{
  get_violins(x, from_groups, to_group, exponentiate)
}
\arguments{
  \item{x}{An object generated by \code{cellmig}.}
  \item{from_groups}{A vector of group names to compare against the 
  target group.}
  \item{to_group}{A single group name serving as the reference for 
  comparisons.}
  \item{exponentiate}{logical, should the effect be exponentiating turning
log-fold-changes into more interpretable fold-changes}
}
\details{
  The function extracts posterior samples of treatment group-level means and 
  computes differences between specified groups. It generates a violin plot 
  illustrating these differences and returns both the computed data and the 
  plot object.
}
\value{
  A list with two elements:
  \item{ds}{A data frame containing computed differences, treatment group 
  identifiers, and associated statistics.}
  \item{plot}{A ggplot2 object representing the violin plot of the computed
  treatment differences.}
}
\examples{
data(d_mini, package = "cellmig")
o <- cellmig(x = d_mini,
             control = list(mcmc_warmup = 200,
                            mcmc_steps = 500,
                            mcmc_chains = 2,
                            mcmc_cores = 2,
                            mcmc_algorithm = "NUTS",
                            adapt_delta = 0.8,
                            max_treedepth = 10))
                            
str(get_groups(x = o))
u <- get_violins(x = o, 
                 from_groups = c("C2|D3", "C2|D4"),
                 to_group = "C3|D3",
                 exponentiate = FALSE)
}
\seealso{
  \code{cellmig}, \code{get_pairs}, \code{get_groups}
}

\name{d}
\alias{d}
\alias{d_mini}
\docType{data}
\title{
Example dataset \code{d}
}
\description{
This dataset \code{d} contains simulated cell migration speed data from an 
imaginery experiment. It includes migration speed values (column \code{v}) 
for different samples (column \code{sample}) treated with chemical compounds 
(column \code{compound}) administered at a dost (column \code{dose}) on
experimental plate with identifier (column \code{plate}). The dataset 
\code{d_mini} contains a subset of \code{d} with only four of the compounds 
(C1 to C4) and three doses (D3, D5, and D5).
}
\usage{
data("d", package = "cellmig")
}
\format{
  A data frame with 6,300 (cells) observations with the following features.
  \describe{
    \item{\code{v}}{a numeric vector with cell migration speeds}
    \item{\code{well}}{a character vector}
    \item{\code{compound}}{a character vector}
    \item{\code{dose}}{a character vector}
    \item{\code{plate}}{a character vector}
    \item{\code{offset}}{indicator: 1 = control, 0 = non-control. Used for
    batch correction}
  }
}
\details{
The code used to generate this data is in  inst/script/sim_s.R
}
\source{
The code used to generate this data is in  inst/script/sim_s.R
}
\references{
The code used to generate this data is in  inst/script/sim_s.R
}
\examples{
data(d, package = "cellmig")
data(d_mini, package = "cellmig")
str(d)
str(d_mini)
}
\keyword{datasets}

\name{cellmig}
\alias{cellmig}
\title{Model-based quantification of cell migration speed}
\description{
The \code{cellmig} function estimates cell migration speed using a 
Bayesian model implemented in \code{rstan}. It takes a data.frame as 
input and allows users to configure the Markov Chain Monte Carlo (MCMC) 
sampling procedure via the \code{control} parameter. The function 
returns a list containing:

\itemize{
\item \code{fit} - the fitted model as an \code{rstan} object.
\item \code{data} - the input/processed input data
\item \code{posteriors} - a summary of model parameters: means, medians, and 
95\% credible intervals.
\item \code{control} - the list of input controls
}
}
\usage{
cellmig(x, control = NULL)
}
\arguments{
\item{x}{A data.frame containing the following columns. Each row represents
the features of a cell:
\itemize{
\item well = character. Well ID (w1, w2, w3, etc.).
\item plate = character. plate ID (p1, p2, p3, etc.). A plate contains 
multiple wells.
\item compound = character. Compound name (c1, c2, c3, etc.)
\item dose = character. Treatment dose (0, 1, 5, 10, low, mid, high, etc.)
\item v = numeric. Observed cell migration speed.
\item offset = binary (0 or 1). Indicates whether a treatment should be used 
for batch correction across plates. Default: 0 (no correction). Set to 1 for 
specific treatment groups used as offsets, ensuring they appear on each plate.
}
}
\item{control}{A list configuring the MCMC sampling algorithm, and parameter
priors with the following default values:
\itemize{
\item mcmc_warmup = 500 (Number of warmup iterations).
\item mcmc_steps = 1500 (Number of sampling iterations).
\item mcmc_chains = 4 (Number of Markov chains).
\item mcmc_cores = 1 (Number of CPU cores used for sampling).
\item mcmc_algorithm = "NUTS" (MCMC algorithm).
\item adapt_delta = 0.8 (Target acceptance probability for NUTS sampler).
\item max_treedepth = 10 (Maximum depth of tree exploration in NUTS sampler).
\item prior_alpha_p_M = 2, prior_alpha_p_SD = 1 (prior on alpha_p: normal
distribution mean (M) and standard deviation (SD))
\item prior_sigma_bio_M = 0, prior_sigma_bio_SD = 0.5 (prior on sigma_bio: 
normal distribution mean (M) and standard deviation (SD))
\item prior_sigma_tech_M = 0, prior_sigma_tech_SD = 0.5 (prior on sigma_tech: 
normal distribution mean (M) and standard deviation (SD))
\item prior_kappa_mu_M = 2, prior_kappa_mu_SD = 1 (prior on kappa_mu: 
normal distribution mean (M) and standard deviation (SD))
\item prior_kappa_sigma_M = 0, prior_kappa_sigma_SD = 1 (prior on kappa_sigma: 
normal distribution mean (M) and standard deviation (SD))
\item prior_mu_group_M = 0, prior_mu_group_SD = 1 (prior on mu_group: 
normal distribution mean (M) and standard deviation (SD))
}
}
}
\value{
A list containing:
\itemize{
\item \code{fit} = The fitted model as an rstan object.
\item \code{data} = The raw and processed input data.
\item \code{posteriors} = A summary of model parameters, including means 
and 95\% credible intervals.
\itemize{
    \item{alpha_p: batch effect on plate p}
    \item{delta_t: overall treatment effects relative to the selected control}
    \item{delta_pt: plate-specific treatment effects relative to the selected 
    control}
    \item{well_mu: mean of cell velocity distribution per well}
    \item{well_kappa: shape of cell velocity distribution per well}
    \item{kappa_mu, kappa_sigma: mean/standard deviation parameters of the
    population of well_kappas}
    \item{sigma_bio: variability between biological replicates}
    \item{sigma_tech: variability between technical replicates}
    \item{yhat: posterior predictions}
    }
\item \code{control} = The list of input controls
}
}
\examples{
data(d, package = "cellmig")
o <- cellmig(x = d,
             control = list(mcmc_warmup = 200,
                            mcmc_steps = 700,
                            mcmc_chains = 2,
                            mcmc_cores = 2,
                            mcmc_algorithm = "NUTS",
                            adapt_delta = 0.8,
                            max_treedepth = 10))
str(o)
}

\name{get_ppc_violins}
\alias{get_ppc_violins}
\title{Posterior Predictive Checks (PPC) Visualization}

\usage{
get_ppc_violins(x, wrap = FALSE, ncol = 4)
}

\arguments{
  \item{x}{The function takes as its main input (\code{x}) the output of the 
\code{cellmig} function, containing posterior samples from a Bayesian model, 
specifically the output of a fitted Stan model with posterior predictive 
samples stored in \code{x$f}.}
\item{wrap}{logical, if wrap = FALSE (default) we will use facet_grid, 
and if wrap = TRUE we will use facet_wrap to split data into compound x 
plate pairs.}
\item{ncol}{In case wrap = TRUE, how many columns should the plot have?}
}

\value{
A \code{ggplot2} object displaying:
\itemize{
  \item Violin plots of posterior predictive distributions for each compound 
  and plate.
  \item Overlaid scatter plots (sina plot) of observed values.
}
}

\description{
The \code{get_ppc_violins} function visualizes posterior predictive checks (PPC) by 
comparing observed data with posterior predictive distributions. The function
extracts simulated data from the fitted Stan model and overlays them with 
observed values, grouped by compound and plate.
}

\details{
The function extracts posterior predictive samples using \code{rstan::extract}, 
reshapes them into long format using \code{reshape2::melt}, and merges them 
with metadata from the input object. It then generates a \code{ggplot2} 
visualization, where:
\itemize{
  \item Violin plots (dashed red) represent posterior predictive distributions.
  \item Overlaid sina plots (black) represent observed data points.
  \item Facets are arranged by compound and plate for better comparison.
}
}

\examples{
data(d_mini, package = "cellmig")
o <- cellmig(x = d_mini,
             control = list(mcmc_warmup = 200,
                            mcmc_steps = 500,
                            mcmc_chains = 2,
                            mcmc_cores = 2,
                            mcmc_algorithm = "NUTS",
                            adapt_delta = 0.8,
                            max_treedepth = 10))

p <- get_ppc_violins(x = o, wrap = TRUE, ncol = 4)
print(p)
}
